﻿
using Produmex.Foundation;
using Produmex.Foundation.Barcode;
using Produmex.Foundation.Data.DbClient;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Generators.BarcodeController;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Data.ViewObjects.Definitions.Views;
using Produmex.Sbo.Logex.KnightonFoods.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.KnightonFoods.Providers;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Globalization;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;

// PmxWorkflowExecutionTypes.SUB_FLOW
namespace WorkflowScript_LiveLoadKnightonFoodsScript
{

    public class WorkflowScript_LiveLoadKnightonFoodsScript : WorkflowInstanceScriptBase
    {
        private const string QUALITY_TYPE = "PMX_LUID_KF";

        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        /// <summary>
        /// The quality values flow
        /// </summary>
        private WorkflowInstanceBase m_enterQualityValuesFlow;

        private WorkflowInstanceBase m_printInvoiceDocumentsSubFlow;
        private Collection<DocumentRef> m_documentRefsSalesInvoiceCreatedHookFlow;


        //sales invoice created hook flow
        private WorkflowInstanceBase m_salesInvoiceCreatedHookFlow;


        // Input parameters
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;

        protected bool m_subFlowBackRequested;

        /// <summary>
        /// The m_enter quality values flow_ list of quality values
        /// </summary>
        protected Collection<QualityValueInfo> m_enterQualityValuesFlow_ListOfQualityValues;

        public WorkflowScript_LiveLoadKnightonFoodsScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory)
            : base(parent, factory)
        {
            m_enterQualityValuesFlow = this.Create("EnterQualityValuesScript");
        }

        /// <summary>
        /// Executes this instance.
        /// </summary>
        protected override void Execute()
        {
            // Parameters in scope
            Session session = GetScopeParameter("Session") as Session;

            ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;

            IBarcodeController barcodeController = null;

            ProdumexError pmxError = null;
            string initialErrorKey = null;
            Produmex.Foundation.Messages.Message msg = null;

            string query = null;

            DataSet currentUnitCount = null;

            string enteredSSCC = null;

            DataSet dsDestinationLocations = null;
            DataSet dsProductsOnSSCC = null;

            PmxOseWarehouseInfo whsInfo = null;

            string liveLoadingCode = null;

            string dstLocationCode = null;
            string dstWarehouseCode = null;

            string srcLocationCode = null;
            string srcWarehouseCode = null;

            BuildQuery.LocationFilter selectLocationFilter = BuildQuery.LocationFilter.All;
            string defaultLocationCode = null;
            string enteredDestinationLocationCode = null;

            double? locationCurrentStock = null;

            int luid;

            string cardCode = null;
            string cardName = null;
            string shipToAddress = null;

            List<LockInformation> salesLockInfo = new List<LockInformation>();

            Collection<DocumentRef> docList = new Collection<DocumentRef>();

            DataSet dsLockedItems = null;


            m_salesInvoiceCreatedHookFlow = Create("SalesInvoiceCreatedHookScript");
            m_salesInvoiceCreatedHookFlow.BindParameters(
                this.DefaultCultureInfo,
                this.PmxOseCompany,
                new ReadOnlyBinder<Collection<DocumentRef>>("DocRefs", this, "m_documentRefsSalesInvoiceCreatedHookFlow")
                );


            sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
                    delegate (PmxDbConnection conn, object[] parameters)
                    {
                        PmxBarcodeProvider bcProv = new PmxBarcodeProvider(conn);
                        barcodeController = bcProv.GetBarcodeController();
                        return null;
                    });


        #region Scan SSCC
        Step_ScanSSCC:

            enteredSSCC = null;
            dsProductsOnSSCC = null;


            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IIdentifyGS1DataScreen),
                this.DefaultCultureInfo, BuildParamCollection(
                "InitialErrorKey", initialErrorKey,
                "TitleKey", "MSG_TITLE_IDENTIFY_SSCC",
                "BarcodeController", barcodeController,
                "EnableForwardButton", true,
                "AllowedButtons", new string[] { "MSG_KF_TRAILER_FULL" }));
            initialErrorKey = null;
            msg = WaitForMessage();

            if (msg.Name.EndsWith(".ButtonClicked"))
            {
                //Ask if user is sure
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                                    DefaultCultureInfo.Get(), BuildParamCollection(
                                        "TitleKey", "MSG_TITLE_CONFIRM_TRAILER_FULL_KF",
                                        "MessageKey", "MSG_QUESTION_CONFIRM_TRAILER_FULL_KF"));
                msg = WaitForMessage();

                if (msg.Name.ToLower().EndsWith(".yes"))
                {
                    goto Step_SelectDock;
                }


                goto Step_ScanSSCC;
            }
            if (msg.Name.EndsWith(".DataEntered"))
            {
                BarcodeContainer barcode = ExtractParameter<BarcodeContainer>(msg.Parameters, "data");
                try
                {
                    barcode = barcodeController.GetScannedData(barcode.FreeBarcode, true, barcode);
                }
                catch
                {
                    // Do nothing
                }

                enteredSSCC = barcode.GS1DataObject.Sscc;


                if (string.IsNullOrWhiteSpace(enteredSSCC))
                {
                    //SSCC not valid
                    initialErrorKey = "MSG_ERROR_INVALID_SSCC";
                    goto Step_ScanSSCC;
                }

                //check if SSCC exists
                query = BuildQuery.GetProductOnSSCC(enteredSSCC, null, sboProviderService.GetDbTool(), sboProviderService.GetDbTool().CultureInfo);
                dsProductsOnSSCC = sboProviderService.RunView(false, null, null, query);

                if (this.GetNumberOfRows(dsProductsOnSSCC) == 0)
                {
                    //SSCC not valid
                    initialErrorKey = "MSG_ERROR_INVALID_SSCC";
                    goto Step_ScanSSCC;
                }

                //check if location is locked
                if (sboProviderService.InvokeMethodWithDbConnection<bool>(false, false, null, null, new DelegateWithPmxDbConnection<bool>(ExecuteIsLocationLocked), dsProductsOnSSCC.Tables[0].Rows[0][PmxInventoryTotalTable.Columns.StorLocCode.NAME] as string))
                {
                    //the location is locked
                    initialErrorKey = "MSG_ERROR_LOCATION_LOCKED";
                    goto Step_ScanSSCC;
                }

                //get warehouse

                whsInfo = sboProviderService.InvokeMethodWithDbConnection<PmxOseWarehouseInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOseWarehouseInfo>(ExecuteGetPmxWarehouseCode), dsProductsOnSSCC.Tables[0].Rows[0][PmxInventoryTotalTable.Columns.StorLocCode.NAME] as string);


                //check if stock on SSCC is allowed for shipping
                query = BuildQuery.GetProductOnSSCCIncorrectQualityStatus(enteredSSCC, null, sboProviderService.GetDbTool(), sboProviderService.GetDbTool().CultureInfo);
                DataSet ds = sboProviderService.RunView(false, null, null, query);

                if (this.GetNumberOfRows(ds) > 0)
                {
                    //SSCC not valid
                    initialErrorKey = "MSG_ERROR_INVALID_SSCC_QUALITY_STATUS";
                    goto Step_ScanSSCC;
                }

                goto Step_IdentifyItemsShipmentHandling;

            }

            return;
        #endregion

        #region Identify items shipment handling
        Step_IdentifyItemsShipmentHandling:

            luid = Convert.ToInt32(dsProductsOnSSCC.Tables[0].Rows[0][PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME]);
            liveLoadingCode = dsProductsOnSSCC.Tables[0].Rows[0][PmxItemTableKnighton.Columns.LiveLoadingType.NAME] as string;
            bool destinationIs3PL = (dsProductsOnSSCC.Tables[0].Rows[0][PmxLiveLoadingDefinitionTableKnighton.Columns.Is3PL.NAME] as string) == "Y" ? true : false;
            foreach (DataRow row in dsProductsOnSSCC.Tables[0].Rows)
            {
                string itemCode = row[SboItemTable.Columns.ItemCode.NAME] as string;
                string itemDestination = row[PmxItemTableKnighton.Columns.LiveLoadingType.NAME] as string;
                bool is3PL = (row[PmxLiveLoadingDefinitionTableKnighton.Columns.Is3PL.NAME] as string) == "Y" ? true : false;
                if (string.IsNullOrEmpty(itemDestination))
                {
                    //show warning for no destination
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                       DefaultCultureInfo.Get(), BuildParamCollection(
                           //TODO: change message
                           "MessageKey", "MSG_KF_ERROR_DESTINATION_IS_NOT_FILLED",
                           "NoTranslationOfMessageKey", false,
                           "Parameters", new object[] { itemCode },
                           "ShowButton", true));
                    WaitForMessage();
                    goto Step_ScanSSCC;

                }
                else if (is3PL != destinationIs3PL)
                {
                    //show warning for different destinations
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                       DefaultCultureInfo.Get(), BuildParamCollection(
                           "MessageKey", "MSG_KF_ERROR_DESTINATIONS_ARE_DIFFERENT_FOR_THE_ITEMS",
                           //TODO: change message
                           "NoTranslationOfMessageKey", false,
                           "ShowButton", true));
                    WaitForMessage();
                    goto Step_ScanSSCC;
                }
            }

            if (destinationIs3PL)
            {
                goto Step_Scan3PLDestinationLocation;
            }
            else
            {
                goto Step_CheckPremiereStock;
            }
        #endregion

        #region Scan 3PL destination location
        Step_Scan3PLDestinationLocation:

            enteredDestinationLocationCode = null;
            query = BuildQuery.GetAllLoadingDocksOfWarehouseForLiveLoading3PL(null, whsInfo.PmxWarehouseCode, liveLoadingCode, BuildQuery.LocationFilter.Loaded, sboProviderService.GetDbTool().CultureInfo);
            dsDestinationLocations = sboProviderService.RunView(false, null, null, query);

            locationCurrentStock = null;

            if (GetNumberOfRows(dsDestinationLocations) > 0)
            {
                defaultLocationCode = dsDestinationLocations.Tables[0].Rows[0][PmxOseDockWithWarehouseViewDefinition.Columns.Code.NAME] as string;
                query = BuildQuery.GetLogisticUnitCountOnLocation(defaultLocationCode, whsInfo.PmxWarehouseCode, sboProviderService.GetDbTool().CultureInfo);
                currentUnitCount = sboProviderService.RunView(false, null, null, query);
                if (GetNumberOfRows(currentUnitCount) > 0)
                {
                    double count = Convert.ToDouble(currentUnitCount.Tables[0].Rows[0][0, DataRowVersion.Default]);
                    if (count > 0)
                    {
                        locationCurrentStock = count;
                    }
                }
            }
            else
            {
                defaultLocationCode = null;
            }
            
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.MoveOrder.IIdentifyLocationScreen),
                                   DefaultCultureInfo.Get(), BuildParamCollection(
                                       "InitialErrorKey", initialErrorKey,
                                       "TitleKey", "MSG_TITLE_SCAN_DESTINATION_LOCATION",
                                       "HideDefaultLocation", false,
                                       "DefaultLocation", defaultLocationCode,
                                       "LocationCurrentStock", locationCurrentStock,
                                       "Uom", base.ResourceStore.GetValue<string>("MSG_KF_HANDLING_UNIT"),
                                       "AllowedButtons", new string[] { "MSG_BUTTON_SELECT_LOCATION", "MSG_BUTTON_SELECT_EMPTY_LOCATION" }
           ));
            msg = WaitForMessage();

            initialErrorKey = null;

            if (msg.Name.EndsWith(".ButtonClicked"))
            {
                switch (ExtractParameter<string>(msg.Parameters, "buttonKey"))
                {
                    case "MSG_BUTTON_SELECT_EMPTY_LOCATION":
                        selectLocationFilter = BuildQuery.LocationFilter.Empty;
                        break;
                    case "MSG_BUTTON_SELECT_LOCATION":
                        selectLocationFilter = BuildQuery.LocationFilter.All;
                        break;
                }
            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
                goto Step_ScanSSCC;
            }
            else
            {
                selectLocationFilter = BuildQuery.LocationFilter.All;
            }

            if (msg.Name.EndsWith(".LocationEntered"))
            {
                dstLocationCode = ExtractParameter<string>(msg.Parameters, "locationCode");

                query = BuildQuery.GetAllLoadingDocksOfWarehouseForLiveLoading3PL(dstLocationCode, whsInfo.PmxWarehouseCode, liveLoadingCode, BuildQuery.LocationFilter.All, sboProviderService.GetDbTool().CultureInfo);
                dsDestinationLocations = sboProviderService.RunView(false, null, null, query);

                if (this.GetNumberOfRows(dsDestinationLocations) != 1)
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", "MSG_ERROR_NO_LOCATION_FOUND",
                            "NoTranslationOfMessageKey", false,
                            "ShowButton", true));
                    WaitForMessage();
                    goto Step_Scan3PLDestinationLocation;
                }
                else
                {
                    goto Step_CheckQualityValues;
                }
            }

            query = BuildQuery.GetAllLoadingDocksOfWarehouseForLiveLoading3PL(enteredDestinationLocationCode, whsInfo.PmxWarehouseCode, liveLoadingCode, selectLocationFilter, sboProviderService.GetDbTool().CultureInfo);
            dsDestinationLocations = sboProviderService.RunView(false, null, null, query);


            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectLocationScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                "InitialErrorKey", initialErrorKey,
                "TitleKey", "MSG_TITLE_SELECT_DESTINATION_LOCATION",
                "LocationDS", dsDestinationLocations,
                "ShowStopButton", false));

            msg = WaitForMessage();
            // Reset latest error
            initialErrorKey = null;

            if (msg.Name.EndsWith(".LocationSelected"))
            {
                dstLocationCode = ExtractParameter<string>(msg.Parameters, "locationCode");

                goto Step_CheckQualityValues;
            }
            //go back
            goto Step_ScanSSCC;
        #endregion

        #region Move 3PL stock
        Step_Move3PLStock:
            try
            {
                docList = sboProviderService.InvokeMethodWithDbConnection<Collection<DocumentRef>>(false, false, null, null, delegate (PmxDbConnection conn, object[] parameters)
                {
                    PmxMoveProvider moveProv = new PmxMoveProvider(conn);
                    return moveProv.MakeAdHocMoveForLUIDs(new Collection<int>() { luid }, dstLocationCode, null, null, null);
                });
                if (docList != null && docList.Count > 0)
                {
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", "MSG_ITEMS_MOVED",
                            "NoTranslationOfMessageKey", false,
                            "ShowButton", true));
                    WaitForMessage();
                }
            }
            catch (Exception ex)
            {
                s_log.Error("", ex);
                if (!(ex is ProdumexException) || ((ProdumexException)ex).Error == null)
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", ExceptionHelper.Format(ex, this.DefaultCultureInfo),
                            "NoTranslationOfMessageKey", true,
                            "IsError", true,
                            "ShowButton", true));
                    WaitForMessage();

                }
                else
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "PmxError", ((ProdumexException)ex).Error,
                            "ShowButton", true));
                    WaitForMessage();
                }

                //retry?
                //ask user if he wants to retry
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "TitleKey", "MSG_TITLE_RETRY_UPDATE",
                        "MessageKey", "MSG_QUESTION_AN_ERROR_OCCURED_WANT_RETRY"));
                msg = WaitForMessage();

                if (msg.Name.ToLower().EndsWith(".yes"))
                {
                    goto Step_Move3PLStock;
                }


                goto Step_ScanSSCC;
            }
            goto Step_ScanSSCC;
        #endregion

        #region Check Premiere stock
        Step_CheckPremiereStock:
            Dictionary<string, double> itemQuantities = new Dictionary<string, double>();
            object o = null;
            Dictionary<string, Dictionary<int, double>> itriQuantities = new Dictionary<string, Dictionary<int, double>>();

            salesLockInfo = new List<LockInformation>();

            foreach (DataRow row in dsProductsOnSSCC.Tables[0].Rows)
            {
                o = row[PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME];

                string itemCode = row[SboItemTable.Columns.ItemCode.NAME] as string;
                double quantity = Convert.ToDouble(row[PmxInventoryTotalTable.Columns.Quantity.NAME]);
                bool is3PL = (row[PmxLiveLoadingDefinitionTableKnighton.Columns.Is3PL.NAME] as string) == "Y" ? true : false;
                if (!itemQuantities.ContainsKey(itemCode))
                {
                    itemQuantities.Add(itemCode, 0);
                }
                itemQuantities[itemCode] += quantity;

                if (!(o is System.DBNull))
                {
                    int itemTransactionalInfoKey = Convert.ToInt32(o);
                    if (!itriQuantities.ContainsKey(itemCode))
                    {
                        itriQuantities.Add(itemCode, new Dictionary<int, double>());
                    }
                    if (!itriQuantities[itemCode].ContainsKey(itemTransactionalInfoKey))
                    {
                        itriQuantities[itemCode].Add(itemTransactionalInfoKey, 0);
                    }
                    itriQuantities[itemCode][itemTransactionalInfoKey] += quantity;
                }
            }
            cardCode = null;
            cardName = null;
            shipToAddress = null;

            foreach (string itemCode in itemQuantities.Keys.ToList())
            {
                double itemQuantity = itemQuantities[itemCode];
                query = BuildQuery.GetOpenSalesOrderLinesWithoutLockedQuantity(itemCode, true, cardCode, shipToAddress, sboProviderService.GetDbTool(), sboProviderService.GetDbTool().CultureInfo);
                DataSet dsOpenSalesOrderLines = sboProviderService.RunView(false, null, null, query);
                if (GetNumberOfRows(dsOpenSalesOrderLines) == 0)
                {
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                       DefaultCultureInfo.Get(), BuildParamCollection(
                           "MessageKey", "MSG_KF_ERROR_NO_OPEN_SALES_ORDER_LINES_FOUND",
                           "NoTranslationOfMessageKey", false,
                           "Parameters", new object[] { itemCode },
                           "ShowButton", true));
                    WaitForMessage();
                    //TODO: add message
                    goto Step_ScanSSCC;
                }
                if (string.IsNullOrEmpty(cardCode) /*|| string.IsNullOrWhiteSpace(shipToAddress)*/)
                {
                    cardCode = dsOpenSalesOrderLines.Tables[0].Rows[0][SboSalesOrderTable.Columns.CardCode.NAME] as string;
                    cardName = dsOpenSalesOrderLines.Tables[0].Rows[0][SboSalesOrderTable.Columns.CardName.NAME] as string;
					//Do not set ship to address here, because it is possible the first row does not have enough stock to assign the stock to
                    //shipToAddress = dsOpenSalesOrderLines.Tables[ 0 ].Rows[ 0 ][ SboSalesOrderLineTable.Columns.ShipToDesc.NAME ] as string;
                }

                if (string.IsNullOrEmpty(cardCode) /*|| string.IsNullOrWhiteSpace(shipToAddress)*/)
                {
                    string docNum = dsOpenSalesOrderLines.Tables[0].Rows[0][SboSalesOrderTable.Columns.DocNum.NAME] as string;
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                       DefaultCultureInfo.Get(), BuildParamCollection(
                           "MessageKey", "MSG_ERROR_DESTINATION_IS_NOT_FILLED_FOR_SALES_ORDER",
                           "NoTranslationOfMessageKey", false,
                           "Parameters", new object[] { docNum },
                           "ShowButton", true));
                    WaitForMessage();
                    //Todo: show message
                    goto Step_ScanSSCC;
                }

                query = BuildQuery.GetOpenSalesOrderLinesWithoutLockedQuantity(itemCode, false, cardCode, shipToAddress, sboProviderService.GetDbTool(), sboProviderService.GetDbTool().CultureInfo);
                dsOpenSalesOrderLines = sboProviderService.RunView(false, null, null, query);
                int lineCount = dsOpenSalesOrderLines.Tables[0].Rows.Count;

                PmxItemInfo itemInfo = sboProviderService.InvokeMethodWithDbConnection<PmxItemInfo>(false, false, null, null, delegate (PmxDbConnection conn, object[] parameters)
                {
                    PmxItemAllConnectionsProvider prov = new PmxItemAllConnectionsProvider(conn);
                    return prov.GetCachedItemInfo(itemCode);
                });

				//Make a grouping of quantity by docentry
				Dictionary<int, double> listOfQuantityByOrder = new Dictionary<int, double>();

				for (int i = 0; i < lineCount; i++)
				{
					DataRow row = dsOpenSalesOrderLines.Tables[0].Rows[i];
					double rowQuantity = Convert.ToDouble(row[SboSalesOrderLineTable.Columns.Quantity.NAME]);
					int docEntry = Convert.ToInt32(row[SboSalesOrderTable.Columns.DocEntry.NAME]);

					if (!listOfQuantityByOrder.ContainsKey(docEntry))
					{
						listOfQuantityByOrder.Add(docEntry, 0);
					}
					listOfQuantityByOrder[docEntry] += rowQuantity;
				}


				for (int i = 0; i < lineCount; i++)
				{
					DataRow row = dsOpenSalesOrderLines.Tables[0].Rows[i];
					double rowQuantity = Convert.ToDouble(row[SboSalesOrderLineTable.Columns.Quantity.NAME]);
					int docEntry = Convert.ToInt32(row[SboSalesOrderTable.Columns.DocEntry.NAME]);

					if (itemQuantities[itemCode] <= listOfQuantityByOrder[docEntry])
					{
						if (string.IsNullOrEmpty(shipToAddress))
						{
							//No ship to address set
							//So this is the first time we assign stock, so now set the ship to address
							shipToAddress = row[SboSalesOrderLineTable.Columns.ShipToDesc.NAME] as string;
						}

						if (shipToAddress == row[SboSalesOrderLineTable.Columns.ShipToDesc.NAME] as string)
						{
							//Only add if ship to address is the same

							//only allow if full pallet can be assigned to the same order
							if (itemQuantities[itemCode] > rowQuantity)
							{
								//Available stock quantity is higher than what is on the sales order, so take the quantity of the sales order
								LockSalesOrderRowQuantities(whsInfo, luid, cardCode, cardName, salesLockInfo, rowQuantity, itriQuantities, itemCode, itemInfo, row);
								itemQuantities[itemCode] -= rowQuantity;
							}
							else
							{
								//Available stock quantity is lower or equal to sales order quantity, so take the stock quantity
								LockSalesOrderRowQuantities(whsInfo, luid, cardCode, cardName, salesLockInfo, itemQuantities[itemCode], itriQuantities, itemCode, itemInfo, row);
								itemQuantities[itemCode] = 0;
							}
						}
					}


					//if (itemQuantities[itemCode] > rowQuantity)
					//{

					//	//if it is the last line
					//	if (i == lineCount - 1)
					//	{
					//		LockSalesOrderRowQuantities(whsInfo, luid, cardCode, cardName, salesLockInfo, itemQuantities[itemCode], itriQuantities, itemCode, itemInfo, row);
					//	}
					//	else
					//	{
					//		LockSalesOrderRowQuantities(whsInfo, luid, cardCode, cardName, salesLockInfo, rowQuantity, itriQuantities, itemCode, itemInfo, row);
					//		itemQuantities[itemCode] -= rowQuantity;
					//	}
					//}
					//else
					//{
					//	LockSalesOrderRowQuantities(whsInfo, luid, cardCode, cardName, salesLockInfo, itemQuantities[itemCode], itriQuantities, itemCode, itemInfo, row);
					//	break;
					//}
				}

				if (itemQuantities[itemCode] > 0)
				{
					//not all quantity assigned to a sales order, so so not allow to proceed
					initialErrorKey = "MSG_ERROR_NO_OPEN_SALES_ORDER";
					goto Step_ScanSSCC;
				}

			}

            goto Step_ScanPremiereDestinationLocation;
        #endregion

        #region Scan Premiere destination location
        Step_ScanPremiereDestinationLocation:

            enteredDestinationLocationCode = null;
            query = BuildQuery.GetAllLoadingDocksOfWarehouseForLiveLoadingPremier(null, whsInfo.PmxWarehouseCode, liveLoadingCode, cardCode, shipToAddress, BuildQuery.LocationFilter.Loaded, sboProviderService.GetDbTool().CultureInfo);
            dsDestinationLocations = sboProviderService.RunView(false, null, null, query);

            locationCurrentStock = null;

            if (GetNumberOfRows(dsDestinationLocations) > 0)
            {
                defaultLocationCode = dsDestinationLocations.Tables[0].Rows[0][PmxOseDockWithWarehouseViewDefinition.Columns.Code.NAME] as string;
                query = BuildQuery.GetLogisticUnitCountOnLocation(defaultLocationCode, whsInfo.PmxWarehouseCode, sboProviderService.GetDbTool().CultureInfo);
                currentUnitCount = sboProviderService.RunView(false, null, null, query);
                double count = Convert.ToDouble(currentUnitCount.Tables[0].Rows[0][0, DataRowVersion.Default]);
                if (count > 0)
                {
                    locationCurrentStock = count;
                }
            }
            else
            {
                defaultLocationCode = null;
            }

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.MoveOrder.IIdentifyLocationScreen),
                                   DefaultCultureInfo.Get(), BuildParamCollection(
                                       "InitialErrorKey", initialErrorKey,
                                       "TitleKey", "MSG_TITLE_SCAN_DESTINATION_LOCATION",
                                       "HideDefaultLocation", false,
                                       "DefaultLocation", defaultLocationCode,
                                       "LocationCurrentStock", locationCurrentStock,
                                       "Uom", base.ResourceStore.GetValue<string>("MSG_KF_HANDLING_UNIT"),
                                       "AllowedButtons", new string[] { "MSG_BUTTON_SELECT_LOCATION", "MSG_BUTTON_SELECT_EMPTY_LOCATION" }
           ));
            msg = WaitForMessage();
            initialErrorKey = null;

            if (msg.Name.EndsWith(".ButtonClicked"))
            {
                switch (ExtractParameter<string>(msg.Parameters, "buttonKey"))
                {
                    case "MSG_BUTTON_SELECT_EMPTY_LOCATION":
                        selectLocationFilter = BuildQuery.LocationFilter.Empty;
                        break;
                    case "MSG_BUTTON_SELECT_LOCATION":
                        selectLocationFilter = BuildQuery.LocationFilter.All;
                        break;
                }
            }
            else if (msg.Name.EndsWith(".BackRequested"))
            {
                goto Step_ScanSSCC;
            }
            else
            {
                selectLocationFilter = BuildQuery.LocationFilter.All;
            }

            if (msg.Name.EndsWith(".LocationEntered"))
            {
                dstLocationCode = ExtractParameter<string>(msg.Parameters, "locationCode");

                query = BuildQuery.GetAllLoadingDocksOfWarehouseForLiveLoadingPremier(dstLocationCode, whsInfo.PmxWarehouseCode, liveLoadingCode, cardCode, shipToAddress, BuildQuery.LocationFilter.All, sboProviderService.GetDbTool().CultureInfo);
                dsDestinationLocations = sboProviderService.RunView(false, null, null, query);

                if (this.GetNumberOfRows(dsDestinationLocations) != 1)
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", "MSG_ERROR_NO_LOCATION_FOUND",
                            "NoTranslationOfMessageKey", false,
                            "ShowButton", true));
                    WaitForMessage();
                    goto Step_ScanPremiereDestinationLocation;
                }
                else
                {

                    //Take the location from the query, so casing matches.
                    dstLocationCode = dsDestinationLocations.Tables[0].Rows[0][PmxOseDockWithWarehouseViewDefinition.Columns.Code.NAME] as string;

                    goto Step_CheckQualityValues;
                }
            }

            query = BuildQuery.GetAllLoadingDocksOfWarehouseForLiveLoadingPremier(enteredDestinationLocationCode, whsInfo.PmxWarehouseCode, liveLoadingCode, cardCode, shipToAddress, selectLocationFilter, sboProviderService.GetDbTool().CultureInfo);
            dsDestinationLocations = sboProviderService.RunView(false, null, null, query);


            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectLocationScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                "InitialErrorKey", initialErrorKey,
                "TitleKey", "MSG_TITLE_SELECT_DESTINATION_LOCATION",
                "LocationDS", dsDestinationLocations,
                "ShowStopButton", false));

            msg = WaitForMessage();
            // Reset latest error
            initialErrorKey = null;

            if (msg.Name.EndsWith(".LocationSelected"))
            {
                dstLocationCode = ExtractParameter<string>(msg.Parameters, "locationCode");

                goto Step_CheckQualityValues;

            }
            //go back
            goto Step_ScanSSCC;
        #endregion

        #region Move premier stock
        Step_MovePremierStock:
            try
            {
                docList = sboProviderService.InvokeMethodWithDbConnection<Collection<DocumentRef>>(false, false, null, null, new DelegateWithPmxDbConnection<Collection<DocumentRef>>(ExecuteMakeMoveAndLockItems), luid, dstLocationCode, salesLockInfo);
                if (docList != null && docList.Count > 0)
                {
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", "MSG_ITEMS_MOVED",
                            "NoTranslationOfMessageKey", false,
                            "ShowButton", true));
                    WaitForMessage();
                }
            }
            catch (Exception ex)
            {
                s_log.Error("", ex);
                if (!(ex is ProdumexException) || ((ProdumexException)ex).Error == null)
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", ExceptionHelper.Format(ex, this.DefaultCultureInfo),
                            "NoTranslationOfMessageKey", true,
                            "IsError", true,
                            "ShowButton", true));
                    WaitForMessage();

                }
                else
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "PmxError", ((ProdumexException)ex).Error,
                            "ShowButton", true));
                    WaitForMessage();
                }

                //retry?
                //ask user if he wants to retry
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "TitleKey", "MSG_TITLE_RETRY_UPDATE",
                        "MessageKey", "MSG_QUESTION_AN_ERROR_OCCURED_WANT_RETRY"));
                msg = WaitForMessage();

                if (msg.Name.ToLower().EndsWith(".yes"))
                {
                    goto Step_MovePremierStock;
                }


                goto Step_ScanSSCC;
            }
            goto Step_ScanSSCC;
        #endregion

        #region Select dock
        Step_SelectDock:

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.MoveOrder.IIdentifyLocationScreen),
                                   DefaultCultureInfo.Get(), BuildParamCollection(
                                       "InitialErrorKey", initialErrorKey,
                                       "TitleKey", "MSG_TITLE_SCAN_LOCATION",
                                       "HideDefaultLocation", false,
                                       "AllowedButtons", new string[] { "MSG_BUTTON_SELECT_LOCATION" }));
            msg = WaitForMessage();
            initialErrorKey = null;

            if (msg.Name.EndsWith(".BackRequested"))
            {
                goto Step_ScanSSCC;
            }
            else if (msg.Name.EndsWith(".LocationEntered"))
            {
                dstLocationCode = ExtractParameter<string>(msg.Parameters, "locationCode");

                query = BuildQuery.GetAllLoadingDocksForTransfer(dstLocationCode, null, sboProviderService.GetDbTool().CultureInfo);
                dsDestinationLocations = sboProviderService.RunView(false, null, null, query);

                if (this.GetNumberOfRows(dsDestinationLocations) == 0)
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", "MSG_ERROR_NO_LOCATION_FOUND",
                            "NoTranslationOfMessageKey", false,
                            "ShowButton", true));
                    WaitForMessage();
                    goto Step_SelectDock;
                }


                //Take the location from the query, so casing matches.
                dstLocationCode = dsDestinationLocations.Tables[0].Rows[0][PmxOseDockWithWarehouseViewDefinition.Columns.Code.NAME] as string;

            }
            else
            {
                query = BuildQuery.GetAllLoadingDocksForTransfer(null, null, sboProviderService.GetDbTool().CultureInfo);
                dsDestinationLocations = sboProviderService.RunView(false, null, null, query);

                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectLocationScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                    "InitialErrorKey", initialErrorKey,
                    "TitleKey", "MSG_TITLE_SELECT_LOCATION",
                    "LocationDS", dsDestinationLocations,
                    "ShowStopButton", false));

                msg = WaitForMessage();

                initialErrorKey = null;

                if (msg.Name.EndsWith(".BackRequested"))
                {
                    goto Step_SelectDock;
                }

                if (msg.Name.EndsWith(".LocationSelected"))
                {
                    dstLocationCode = ExtractParameter<string>(msg.Parameters, "locationCode");
                    string whsCode = ExtractParameter<string>(msg.Parameters, "warehouseCode");

                    query = BuildQuery.GetAllLoadingDocksForTransfer(dstLocationCode, whsCode, sboProviderService.GetDbTool().CultureInfo);
                    dsDestinationLocations = sboProviderService.RunView(false, null, null, query);

                    if (this.GetNumberOfRows(dsDestinationLocations) != 1)
                    {
                        // Inform the user with the error
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "MessageKey", "MSG_ERROR_NO_LOCATION_FOUND",
                                "NoTranslationOfMessageKey", false,
                                "ShowButton", true));
                        WaitForMessage();
                        goto Step_ScanSSCC;
                    }
                }
            }

            srcLocationCode = dsDestinationLocations.Tables[0].Rows[0][PmxOseDockWithWarehouseViewDefinition.Columns.Code.NAME] as string;
            //pmx code
            srcWarehouseCode = dsDestinationLocations.Tables[0].Rows[0][LocationDataSet.Columns.WAREHOUSECODE.Name] as string;


            destinationIs3PL = (dsDestinationLocations.Tables[0].Rows[0][PmxLiveLoadingDefinitionTableKnighton.Columns.Is3PL.NAME] as string) == "Y";
            if (destinationIs3PL)
            {
                dstLocationCode = dsDestinationLocations.Tables[0].Rows[0][PmxLiveLoadingDefinitionTableKnighton.Columns.DockCode.NAME] as string;
                dstWarehouseCode = dsDestinationLocations.Tables[0].Rows[0][PmxLiveLoadingDefinitionTableKnighton.Columns.SboWarehouseCode.NAME] as string;
                goto Step_Book3PLStock;
            }
            else
            {
                goto Step_BookPremierStock;
            }

            //go back
            goto Step_ScanSSCC;
        #endregion

        #region Book 3PL stock
        Step_Book3PLStock:
            List<int> luids = new List<int>();

            query = BuildQuery.GetLuidsOnLocation(srcLocationCode, srcWarehouseCode, sboProviderService.GetDbTool().CultureInfo);
            DataSet dsLuids = sboProviderService.RunView(false, null, null, query);

            foreach (DataRow row in dsLuids.Tables[0].Rows)
            {
                int tempLuid = Convert.ToInt32(row[PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME]);
                luids.Add(tempLuid);
            }

            try
            {
                docList = sboProviderService.InvokeMethodWithDbConnection<Collection<DocumentRef>>(false, null, null, delegate (SboConnection conn, object[] parameters)
                {
                    PmxMoveProvider moveProv = new PmxMoveProvider(conn);

                    IList<MoveItemInfo> list = moveProv.AddLuidsToMoveItemInfoList(luids);

                    if (list != null)
                    {
                        foreach (MoveItemInfo move in list)
                        {
                            move.DestinationLocationCode = dstLocationCode;
                        }
                    }

                    //make normal move
                    return moveProv.MakeAdHocMove(list, null, dstLocationCode);
                    //return moveProv.MakeAdHocMoveForLUIDs(luids, dstLocationCode, null, null, null);
                });
            }
            catch (Exception ex)
            {
                s_log.Error("", ex);
                if (!(ex is ProdumexException) || ((ProdumexException)ex).Error == null)
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", ExceptionHelper.Format(ex, this.DefaultCultureInfo),
                            "NoTranslationOfMessageKey", true,
                            "IsError", true,
                            "ShowButton", true));
                    WaitForMessage();

                }
                else
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "PmxError", ((ProdumexException)ex).Error,
                            "ShowButton", true));
                    WaitForMessage();
                }

                //retry?
                //ask user if he wants to retry
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "TitleKey", "MSG_TITLE_RETRY_UPDATE",
                        "MessageKey", "MSG_QUESTION_AN_ERROR_OCCURED_WANT_RETRY"));
                msg = WaitForMessage();

                if (msg.Name.ToLower().EndsWith(".yes"))
                {
                    goto Step_Book3PLStock;
                }


                goto Step_ScanSSCC;
            }

            {
                bool proceed = false;
                bool askReprint;
                while (!proceed)
                {
                    int numberOfReports = 0;

                    try
                    {

                        foreach (DocumentRef docRef in docList)
                        {

                            //trigger event
                            numberOfReports += sboProviderService.InvokeMethodWithDbConnection<int>(false, true, null, null, delegate (PmxDbConnection conn, object[] parameters)
                            {

                                PmxPrintReportProvider printProv = new PmxPrintReportProvider(conn);
                                return printProv.PrintReportsForWarehouseMoveAddedEvent(null, session.DeviceInstanceId, docRef, false, out askReprint);
                            });
                        }//foreach
                    }
                    catch (Produmex.Foundation.ProdumexException ex)
                    {
                        s_log.Error(ex.ToString());
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "InitialErrorKey", initialErrorKey,
                                "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                                "MessageAddendum", ex.Message,
                                "PmxError", ex.Error,
                                "ShowButton", true));

                        msg = WaitForMessage();
                    }
                    catch (Exception ex)
                    {
                        s_log.Error(ex.ToString());
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "InitialErrorKey", initialErrorKey,
                                "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                                "MessageAddendum", ex.Message,
                                "PmxError", Produmex.Sbo.Logex.Data.ProdumexErrors.UnexpectedError(s_log, ex),
                                "ShowButton", true));

                        msg = WaitForMessage();
                    }

                    if (numberOfReports > 0)
                    {
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "MessageKey", "MSG_QUESTION_REPRINT_DOCUMENT",
                                "TitleKey", "MSG_TITLE_REPRINT_DOCUMENT"
                                ));

                        msg = WaitForMessage();

                        if (msg.Name.EndsWith(".No"))
                        {
                            proceed = true;
                        }

                    }
                    else
                    {
                        proceed = true;
                    }

                }
            }

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
            DefaultCultureInfo.Get(), BuildParamCollection(
                "MessageKey", "MSG_ITEMS_MOVED",
                "NoTranslationOfMessageKey", false,
                "ShowButton", true));
            WaitForMessage();

            goto Step_ScanSSCC;
        #endregion

        #region Book Premier stock
        Step_BookPremierStock:

            query = BuildQuery.GetLockedItemsForLocation(srcLocationCode, sboProviderService.GetDbTool().CultureInfo);
            dsLockedItems = sboProviderService.RunView(false, null, null, query);
            List<DocumentRef> documents = null;
            try
            {
                documents = sboProviderService.InvokeMethodWithDbConnection<List<DocumentRef>>(false, null, null, new DelegateWithSboConnection<List<DocumentRef>>(ExecuteCreateSalesDeliverNotes), dsLockedItems, sboProviderService.GetDbTool().CultureInfo);
            }
            catch (Exception ex)
            {
                s_log.Error("", ex);
                if (!(ex is ProdumexException) || ((ProdumexException)ex).Error == null)
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", ExceptionHelper.Format(ex, this.DefaultCultureInfo),
                            "NoTranslationOfMessageKey", true,
                            "IsError", true,
                            "ShowButton", true));
                    WaitForMessage();

                }
                else
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "PmxError", ((ProdumexException)ex).Error,
                            "ShowButton", true));
                    WaitForMessage();
                }

                //retry?
                //ask user if he wants to retry
                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                                    DefaultCultureInfo.Get(), BuildParamCollection(
                                        "TitleKey", "MSG_TITLE_RETRY_UPDATE",
                                        "MessageKey", "MSG_QUESTION_AN_ERROR_OCCURED_WANT_RETRY"));
                msg = WaitForMessage();

                if (msg.Name.ToLower().EndsWith(".yes"))
                {
                    goto Step_BookPremierStock;
                }


                goto Step_ScanSSCC;
            }

            {
                bool proceed = false;
                bool askReprint;
                while (!proceed)
                {
                    int numberOfReports = 0;

                    try
                    {
                        foreach (DocumentRef documentRef in documents)
                        {
                            //trigger event
                            numberOfReports += sboProviderService.InvokeMethodWithDbConnection<int>(false, true, null, null, delegate (PmxDbConnection conn, object[] parameters)
                            {

                                PmxPrintReportProvider printProv = new PmxPrintReportProvider(conn);

                                return printProv.PrintReportsForEvent(PmxPrintReportEventType.ShippingSalesDeliveryNoteCreated,
                                         srcLocationCode, session.DeviceInstanceId, documentRef.DocEntry, out askReprint);
                            });
                        }
                    }
                    catch (Produmex.Foundation.ProdumexException ex)
                    {
                        s_log.Error(ex.ToString());
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "InitialErrorKey", initialErrorKey,
                                "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                                "MessageAddendum", ex.Message,
                                "PmxError", ex.Error,
                                "ShowButton", true));

                        msg = WaitForMessage();
                    }
                    catch (Exception ex)
                    {
                        s_log.Error(ex.ToString());
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "InitialErrorKey", initialErrorKey,
                                "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                                "MessageAddendum", ex.Message,
                                "PmxError", Produmex.Sbo.Logex.Data.ProdumexErrors.UnexpectedError(s_log, ex),
                                "ShowButton", true));

                        msg = WaitForMessage();
                    }

                    if (numberOfReports > 0)
                    {
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "MessageKey", "MSG_QUESTION_REPRINT_DOCUMENT",
                                "TitleKey", "MSG_TITLE_REPRINT_DOCUMENT"
                                ));

                        msg = WaitForMessage();

                        if (msg.Name.EndsWith(".No"))
                        {
                            proceed = true;
                        }

                    }
                    else
                    {
                        proceed = true;
                    }

                }
            }

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
            DefaultCultureInfo.Get(), BuildParamCollection(
                "MessageKey", "MSG_ITEMS_MOVED",
                "NoTranslationOfMessageKey", false,
                "ShowButton", true));
            WaitForMessage();

            goto Step_AutoCreateInvoice;

            #endregion

            //Automatic shipping

            Step_AutoCreateInvoice:

            #region AUTO CREATE INVOICE

            Collection<DocumentRef> listOfSalesInvoices = null;
            //nrOfDocsPrinted = 0;

            DocumentRef[] deliveryDocRefs = documents.ToArray();

            //auto create invoice
            if (deliveryDocRefs != null &&
                deliveryDocRefs.Length > 0)
            {
                DocumentRef[] deliveryDocRefsAutomaticInvoicing = sboProviderService.InvokeMethodWithDbConnection<DocumentRef[]>(true, false, null, null,
                    delegate (PmxDbConnection conn, object[] parameters)
                    {
                        PmxSalesDeliveryNoteProvider sdProv = new PmxSalesDeliveryNoteProvider(conn);
                        return sdProv.GetDocumentRefsForAutomaticInvoice(deliveryDocRefs);
                    });

                try
                {
                    if (deliveryDocRefsAutomaticInvoicing != null &&
                        deliveryDocRefsAutomaticInvoicing.Length > 0)
                    {
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                         this.DefaultCultureInfo, BuildParamCollection(
                         "MessageKey", "MSG_ADDING_SALES_INVOICE"));


                        listOfSalesInvoices = sboProviderService.InvokeMethodWithDbConnection<Collection<DocumentRef>>(true, false, null, null,
                         delegate (PmxDbConnection conn, object[] parameters)
                         {
                             PmxSalesInvoiceProvider prov = new PmxSalesInvoiceProvider(conn);
                             return prov.CreateInvoiceFromDelivery(deliveryDocRefsAutomaticInvoicing);
                         });

                    }
                    goto Step_PrintInvoiceDocuments;

                }
                catch (Produmex.Foundation.ProdumexException ex)
                {
                    s_log.Error(ex.ToString());
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "InitialErrorKey", initialErrorKey,
                            "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                            "MessageAddendum", ex.Message,
                            "PmxError", ex.Error,
                            "ShowButton", true));

                    msg = WaitForMessage();
                }
                catch (Exception ex)
                {
                    s_log.Error(ex.ToString());
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "InitialErrorKey", initialErrorKey,
                            "MessageKey", KnownMessageKeys.MSG_ERROR_INTERNAL_ERROR,
                            "MessageAddendum", ex.Message,
                            "PmxError", Produmex.Sbo.Logex.Data.ProdumexErrors.UnexpectedError(s_log, ex),
                            "ShowButton", true));

                    msg = WaitForMessage();
                }

               


                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "TitleKey", "MSG_TITLE_ERROR_TRY_AGAIN",
                        "MessageKey", "MSG_QUESTION_ERROR_TRY_AGAIN"));
                msg = WaitForMessage();

                if (msg.Name.EndsWith(".Yes"))
                {
                    goto Step_AutoCreateInvoice;
                }

               
            }

            goto Step_PrintInvoiceDocuments;

        #endregion

        Step_PrintInvoiceDocuments:

            #region PRINT SALES INVOICE DOCUMENTS

            if (listOfSalesInvoices != null &&
                listOfSalesInvoices.Count > 0)
            {
                try
                {
                    m_printInvoiceDocumentsSubFlow = this.Create("PrintSalesInvoiceDocumentsScript");
                    m_printInvoiceDocumentsSubFlow.BindParameters(
                        this.DefaultCultureInfo,
                        this.PmxOseCompany,
                        new ReadOnlyBinder<Collection<DocumentRef>>("DocRefs", listOfSalesInvoices),
                        new ReadOnlyBinder<string>("LocationCode", srcLocationCode));
                    this.StartSubFlowAndWait(m_printInvoiceDocumentsSubFlow, true);
                }
                catch (Exception ex)
                {
                    s_log.Error("", ex);
                    if (!(ex is ProdumexException) || ((ProdumexException)ex).Error == null)
                    {
                        // Inform the user with the error
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
                                "NoTranslationOfMessageKey", true,
                                "IsError", true,
                                "ShowButton", true));
                        WaitForMessage();

                    }
                    else
                    {
                        // Inform the user with the error
                        session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "PmxError", ((ProdumexException)ex).Error,
                                "ShowButton", true));
                        WaitForMessage();
                    }
                }

                goto Step_SalesInvoiceCreatedHook;

            }

            //no invoice created
            goto Step_ScanSSCC;

        #endregion


        Step_SalesInvoiceCreatedHook:

            #region SALES INVOICE CREATED HOOK

            //sales invoice hook flow
            m_documentRefsSalesInvoiceCreatedHookFlow = listOfSalesInvoices;

            try
            {
                //call flow
                this.StartSubFlowAndWait(m_salesInvoiceCreatedHookFlow, true);
            }
            catch (Exception ex)
            {
                s_log.Error("", ex);
                if (!(ex is ProdumexException) || ((ProdumexException)ex).Error == null)
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
                            "NoTranslationOfMessageKey", true,
                            "IsError", true,
                            "ShowButton", true));
                    WaitForMessage();

                }
                else
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "PmxError", ((ProdumexException)ex).Error,
                            "ShowButton", true));
                    WaitForMessage();
                }
            }



            goto Step_ScanSSCC;
        #endregion

        #region Step_CheckQualityValues
        Step_CheckQualityValues:

            query = BuildQuery.GetLogisticUnitCountOnLocation(dstLocationCode, whsInfo.PmxWarehouseCode, sboProviderService.GetDbTool().CultureInfo);
            currentUnitCount = sboProviderService.RunView(false, null, null, query);
            if (GetNumberOfRows(currentUnitCount) == 0 ||
                Convert.ToInt32( currentUnitCount.Tables[0].Rows[0][0]) == 0)
            {

                m_enterQualityValuesFlow.BindParameters(
                     this.DefaultCultureInfo,
                     this.PmxOseCompany,
                     new ReadOnlyBinder<PmxMomentOfCapture>("MomentOfCapture", PmxMomentOfCapture.Start),
                     new ReadOnlyBinder<string>("BaseType", QUALITY_TYPE),
                     new ReadOnlyBinder<int?>("BaseEntry", luid),
                     new ReadOnlyBinder<bool>("OnlyAskWhenNotRegisteredYet", false),
                     new ReadOnlyBinder<bool>("SaveData", false),

                     new ReadWriteBinder<bool>("BackRequested", this, "m_subFlowBackRequested"),
                     new ReadWriteBinder<Collection<QualityValueInfo>>("ListOfQualityValues", this, "m_enterQualityValuesFlow_ListOfQualityValues")

                     );
                m_subFlowBackRequested = false;
                this.StartSubFlowAndWait(m_enterQualityValuesFlow, true);

                if (m_subFlowBackRequested)
                {
                    if (destinationIs3PL)
                    {
                        goto Step_Scan3PLDestinationLocation;
                    }
                    else
                    {
                        goto Step_ScanPremiereDestinationLocation;
                    }

                }

                if (m_enterQualityValuesFlow_ListOfQualityValues.Count > 0) {

                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "TitleKey", "MSG_TITLE_ASK_USE_TRAILER",
                            "MessageKey", "MSG_KF_QUESTION_USE_TRAILER"));
                    msg = WaitForMessage();

                    if (msg.Name.ToLower().EndsWith(".yes"))
                    {
                        sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null,
                                delegate (PmxDbConnection conn, object[] parameters)
                                {
                                    PmxQualityValueProvider qvProv = new PmxQualityValueProvider(conn);

                                    foreach (QualityValueInfo qvInfo in m_enterQualityValuesFlow_ListOfQualityValues)
                                    {
                                        using (PmxQualityValue qv = qvProv.GetNewBO())
                                        {
                                            qv.BaseEntry = luid;
                                            qv.BaseType = QUALITY_TYPE;
                                            qv.MomentOfCapture = qvInfo.MomentOfCapture;
                                            qv.QualityTypeCode = qvInfo.QualityTypeCode;
                                            qv.QualityValue = qvInfo.QualityValue;
                                            qv.ValueConvertor = qvInfo.ValueConvertor;

                                            qvProv.AddBO(qv);
                                        }
                                    }


                                    return null;
                                }
                                );


                    }
                    else
                    {
                        if (destinationIs3PL)
                        {
                            goto Step_Scan3PLDestinationLocation;
                        }
                        else
                        {
                            goto Step_ScanPremiereDestinationLocation;
                        }
                    }

                }
            }
            if (destinationIs3PL)
            {
                goto Step_Move3PLStock;
            }
            else
            {
                goto Step_MovePremierStock;
            }
            #endregion
        }

        private static void LockSalesOrderRowQuantities(PmxOseWarehouseInfo whsInfo, int luid, string cardCode, string cardName, List<LockInformation> salesLockInfo, double itemQuantity, Dictionary<string, Dictionary<int, double>> itriQuantities, string itemCode, PmxItemInfo itemInfo, DataRow row)
        {
            if (itriQuantities.ContainsKey(itemCode))
            {
                while (itemQuantity > 0)
                {
                    int itriKey = itriQuantities[itemCode].Keys.First();
                    double itriQuantity = itriQuantities[itemCode][itriKey];
                    double quantityToLock;
                    if (itriQuantity >= itemQuantity)
                    {
                        quantityToLock = itemQuantity;
                        itriQuantities[itemCode][itriKey] -= itemQuantity;
                        itemQuantity = 0;
                    }
                    else
                    {
                        quantityToLock = itriQuantity;
                        itriQuantities[itemCode].Remove(itriKey);
                        itemQuantity -= itriQuantity;
                    }
                    salesLockInfo.Add(new LockInformation(
                          Convert.ToInt32(row[PmxSalesOrderTable.Columns.DocEntry.NAME]),
                          Convert.ToInt32(row[PmxSalesOrderLineTable.Columns.LineNum.NAME]),
                          itemInfo,
                          quantityToLock,
                          cardCode,
                          cardName,
                          luid,
                          whsInfo.PmxWarehouseCode,
                          itriKey));
                }
            }
            else
            {
                salesLockInfo.Add(new LockInformation(
                    Convert.ToInt32(row[PmxSalesOrderTable.Columns.DocEntry.NAME]),
                    Convert.ToInt32(row[PmxSalesOrderLineTable.Columns.LineNum.NAME]),
                    itemInfo,
                    itemQuantity,
                    cardCode,
                    cardName,
                    luid,
                    whsInfo.PmxWarehouseCode,
                    null));
            }
        }

        private List<DocumentRef> ExecuteCreateSalesDeliverNotes(SboConnection conn, object[] parameters)
        {
            KFSalesDeliveryNoteProvider prov = new KFSalesDeliveryNoteProvider(conn);

            DataSet dsLockInfo = (DataSet)parameters[0];

            Dictionary<int, int> luids = new Dictionary<int, int>();

            foreach (DataRow row in dsLockInfo.Tables[0].Rows)
            {
                int luid = Convert.ToInt32(row[PmxInventoryLockingDetailTable.Columns.LogUnitIdentKey.NAME]);
                if (!luids.ContainsKey(luid))
                {
                    luids.Add(luid, luid);
                }
            }

            string query = BuildQuery.GetRemoveInventoryLockingByLuidList(luids.Keys.ToList(), (CultureInfo)parameters[1]);
            SboRecordsetHelper.RunQueryNoReturn(s_log, query, conn);
            return prov.AddSalesNoteDeliveryForSalesOrderLines(dsLockInfo);
        }

        private static Collection<DocumentRef> ExecuteMakeMoveAndLockItems(PmxDbConnection conn, object[] parameters)  //luid, dstLocation, LockInfo
        {
            PmxMoveProvider moveProv = new PmxMoveProvider(conn);
            Collection<DocumentRef> documents = moveProv.MakeAdHocMoveForLUIDs(new Collection<int>() { (int)parameters[0] }, (string)parameters[1], null, null, null);

            List<LockInformation> lockingInformations = (List<LockInformation>)parameters[2];

            PmxInventoryLockingDetailProvider lockProv = new PmxInventoryLockingDetailProvider(conn);

            string dstLocation = (string)parameters[1];

            foreach (LockInformation lockInfo in lockingInformations)
            {
                using (PmxInventoryLockingDetail lockDetail = lockProv.GetNewBO())
                {
                    lockDetail.CardCode = lockInfo.CardCode;
                    lockDetail.CardName = lockInfo.CardName;
                    lockDetail.InventoryLockingLevel = PmxInventoryLockingLevel.ItemLUID;
                    lockDetail.InventoryLockStatusCode = PmxInventoryLockingStatusCodes.RESERVED_CODE;
                    lockDetail.ItemCode = lockInfo.ItemInfo.ItemCode;
                    lockDetail.ItemTransactionalInfoKey = lockInfo.ItemTransactionalInfoKey;
                    lockDetail.LogisticUnitIdentKey = lockInfo.LogisticUnitId;
                    lockDetail.PmxWhsCode = lockInfo.PmxWhsCode;
                    lockDetail.QualityStatusCode = PmxQualityStatusCodes.RELEASED_CODE;
                    lockDetail.Quantity = lockInfo.Quantity;
                    lockDetail.StorageLocationCode = dstLocation;

                    lockDetail.QuantityUom2 = PmxItemAllConnectionsProvider.CalculateWeightFromPieces(lockDetail.Quantity, lockInfo.ItemInfo);
                    lockDetail.Uom2 = lockInfo.ItemInfo.Uom2;

                    lockDetail.BaseEntry = lockInfo.DocEntry;
                    lockDetail.BaseType = ((int)SboLinkDocumentType.SalesOrder).ToString();
                    lockDetail.BaseLine = lockInfo.LineNo;

                    lockProv.AddBO(lockDetail);
                }
            }
            return documents;
        }

        /// <summary>
        /// Executes the get PMX warehouse code.
        /// </summary>
        /// <param name="conn">The conn.</param>
        /// <param name="parameters">The parameters.</param>
        /// <returns></returns>
        private static PmxOseWarehouseInfo ExecuteGetPmxWarehouseCode(PmxDbConnection conn, object[] parameters)
        {
            PmxOseProvider oseProv = new PmxOseProvider(conn);
            return oseProv.GetPmxWarehouseInfo((string)parameters[0]);
        }

        /// <summary>
        /// Gets the number of rows.
        /// </summary>
        /// <param name="ds">The ds.</param>
        /// <returns></returns>
        private int GetNumberOfRows(DataSet ds)
        {
            if (ds != null &&
                ds.Tables.Count > 0)
            {
                return ds.Tables[0].Rows.Count;
            }

            return 0;
        }

        private static bool ExecuteIsLocationLocked(PmxDbConnection conn, object[] parameters)
        {
            PmxOseStorageLocationProvider prov = new PmxOseStorageLocationProvider(conn);
            return prov.IsLocationLocked((string)parameters[0]);
        }
    }


    internal struct LockInformation
    {
        public int DocEntry;
        public int LineNo;
        public PmxItemInfo ItemInfo;
        public double Quantity;
        public int LogisticUnitId;
        public string PmxWhsCode;
        public string CardCode;
        public string CardName;
        public int? ItemTransactionalInfoKey;

        public LockInformation(int docEntry, int lineNo, PmxItemInfo itemInfo, double quantity, string cardCode, string cardName, int logisticUnitId, string pmxWhsCode, int? itemTransactionalInfoKey)
        {
            this.DocEntry = docEntry;
            this.LineNo = lineNo;
            this.ItemInfo = itemInfo;
            this.Quantity = quantity;
            this.LogisticUnitId = logisticUnitId;
            this.PmxWhsCode = pmxWhsCode;
            this.CardCode = cardCode;
            this.CardName = cardName;
            this.ItemTransactionalInfoKey = itemTransactionalInfoKey;
        }
    }

    #region BUILD QUERY

    /// <summary>
    /// Static class grouping our native queries
    /// </summary>
    internal static class BuildQuery
    {
        internal enum LocationFilter
        {
            All = 0,
            Empty = 1,
            Loaded = 2,
        }

        /// <summary>
        /// Gets all loading docks of a warehouse for live loading.
        /// </summary>
        /// <param name="oseWarehouseCode">The ose warehouse code.</param>
        /// <param name="liveLoadingCode">The live loading code.</param>
        /// <param name="cultureDatabase">The culture database.</param>
        /// <returns></returns>
        internal static string GetAllLoadingDocksOfWarehouseForLiveLoading3PL(string locationCode, string oseWarehouseCode, string liveLoadingCode, LocationFilter filter, CultureInfo cultureDatabase)
        {
            StringBuilder strBuilder = new StringBuilder();
            strBuilder.Append("SELECT DISTINCT ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Code.NAME, PmxOseDockWithWarehouseViewDefinition.Columns.Code.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Name.NAME, LocationDataSet.Columns.DESCRIPTION.Name);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, LocationDataSet.Columns.WAREHOUSECODE.Name);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, LocationDataSet.Columns.WAREHOUSENAME.Name);
            strBuilder.Append("\n FROM ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
            strBuilder.Append("\n INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseDockTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Code.NAME);
            strBuilder.Append("\n INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Code.NAME);
            strBuilder.Append("\n INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME);


            strBuilder.Append("\n LEFT JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);

            strBuilder.Append("\n LEFT JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, SboItemTable.Columns.ItemCode.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME);

            strBuilder.Append("\n WHERE ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseDockTable.Columns.IsLoadingDock.NAME));
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, cultureDatabase));
            strBuilder.Append("\n AND ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.IsActive.NAME);
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, cultureDatabase));
            strBuilder.Append("\n AND ( ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME);
            strBuilder.Append(" IS NULL OR (");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTableKnighton.NAME, PmxItemTableKnighton.Columns.LiveLoadingType.NAME);
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(liveLoadingCode, cultureDatabase));
            strBuilder.Append(" ))");

            if (!string.IsNullOrWhiteSpace(locationCode))
            {
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Code.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, cultureDatabase));
            }

            if (filter == LocationFilter.Empty)
            {
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, SboItemTable.Columns.ItemCode.NAME);
                strBuilder.Append(" IS NULL ");
            }

            if (filter == LocationFilter.Loaded)
            {
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);
                strBuilder.Append(" IS NOT NULL ");
            }

            if (!string.IsNullOrEmpty(oseWarehouseCode))
            {
                strBuilder.Append(" AND ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(oseWarehouseCode, cultureDatabase));
            }

            strBuilder.Append(" ORDER BY ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Name.NAME);

            return strBuilder.ToString();
        }

        /// <summary>
        /// Gets all loading docks of a warehouse for live loading.
        /// </summary>
        /// <param name="oseWarehouseCode">The ose warehouse code.</param>
        /// <param name="liveLoadingCode">The live loading code.</param>
        /// <param name="cultureDatabase">The culture database.</param>
        /// <returns></returns>
        internal static string GetAllLoadingDocksOfWarehouseForLiveLoadingPremier(string locationCode, string oseWarehouseCode, string liveLoadingCode, string cardCode, string shipToAddress, LocationFilter filter, CultureInfo cultureDatabase)
        {
            StringBuilder strBuilder = new StringBuilder();
            strBuilder.Append("SELECT DISTINCT ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Code.NAME, PmxOseDockWithWarehouseViewDefinition.Columns.Code.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Name.NAME, LocationDataSet.Columns.DESCRIPTION.Name);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, LocationDataSet.Columns.WAREHOUSECODE.Name);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, LocationDataSet.Columns.WAREHOUSENAME.Name);
            strBuilder.Append("\n FROM ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
            strBuilder.Append("\n INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseDockTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Code.NAME);
            strBuilder.Append("\n INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Code.NAME);
            strBuilder.Append("\n INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME);


            strBuilder.Append("\n LEFT JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);

            strBuilder.Append("\n LEFT JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, SboItemTable.Columns.ItemCode.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME);

            strBuilder.Append("\n LEFT JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryLockingDetailTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.LogUnitIdentKey.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME);

            strBuilder.Append("\n LEFT JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(SboSalesOrderLineTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, SboSalesOrderLineTable.NAME, SboSalesOrderLineTable.Columns.DocEntry.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.BaseEntry.NAME);
            strBuilder.Append(" AND ");
            SqlCommandHelper.AddTableColumnName(strBuilder, SboSalesOrderLineTable.NAME, SboSalesOrderLineTable.Columns.LineNum.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.BaseLine.NAME);

            strBuilder.Append("\n LEFT JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(SboSalesOrderTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, SboSalesOrderTable.NAME, SboSalesOrderTable.Columns.DocEntry.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.BaseEntry.NAME);

            strBuilder.Append("\n WHERE ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseDockTable.Columns.IsLoadingDock.NAME));
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, cultureDatabase));
            strBuilder.Append("\n AND ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.IsActive.NAME);
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, cultureDatabase));
            strBuilder.Append("\n AND ( ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME);
            strBuilder.Append(" IS NULL OR (");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTableKnighton.NAME, PmxItemTableKnighton.Columns.LiveLoadingType.NAME);
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(liveLoadingCode, cultureDatabase));
            strBuilder.Append(" ))");

            if (!string.IsNullOrEmpty(cardCode))
            {
                strBuilder.Append(" AND ");
                if (filter == LocationFilter.All || filter == LocationFilter.Empty)
                {
                    strBuilder.Append(" ( ");
                }
                SqlCommandHelper.AddTableColumnName(strBuilder, SboSalesOrderTable.NAME, SboSalesOrderTable.Columns.CardCode.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(cardCode, cultureDatabase));
                if (filter == LocationFilter.All || filter == LocationFilter.Empty)
                {
                    strBuilder.Append(" OR ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, SboSalesOrderTable.NAME, SboSalesOrderTable.Columns.CardCode.NAME);
                    strBuilder.Append(" IS NULL) ");
                }
            }
            if (!string.IsNullOrEmpty(shipToAddress))
            {
                strBuilder.Append(" AND ");
                if (filter == LocationFilter.All || filter == LocationFilter.Empty)
                {
                    strBuilder.Append(" ( ");
                }
                SqlCommandHelper.AddTableColumnName(strBuilder, SboSalesOrderLineTable.NAME, SboSalesOrderLineTable.Columns.ShipToDesc.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(shipToAddress, cultureDatabase));
                if (filter == LocationFilter.All || filter == LocationFilter.Empty)
                {
                    strBuilder.Append(" OR ");
                    SqlCommandHelper.AddTableColumnName(strBuilder, SboSalesOrderLineTable.NAME, SboSalesOrderLineTable.Columns.ShipToDesc.NAME);
                    strBuilder.Append(" IS NULL) ");
                }
            }

            if (!string.IsNullOrWhiteSpace(locationCode))
            {
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Code.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, cultureDatabase));
            }

            if (filter == LocationFilter.Empty)
            {
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, SboItemTable.Columns.ItemCode.NAME);
                strBuilder.Append(" IS NULL ");
            }

            if (filter == LocationFilter.Loaded)
            {
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);
                strBuilder.Append(" IS NOT NULL ");
            }

            if (!string.IsNullOrEmpty(oseWarehouseCode))
            {
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(oseWarehouseCode, cultureDatabase));
            }

            strBuilder.Append(" ORDER BY ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Code.NAME);

            return strBuilder.ToString();
        }


        /// <summary>
        /// Command for getting all products order by name
        /// </summary>
        /// <param name="sscc">The SSCC.</param>
        /// <param name="locationCode">The location code.</param>
        /// <param name="dbTool">The db tool.</param>
        /// <returns>
        /// Command for getting all products order by name
        /// </returns>
        public static string GetProductOnSSCC(string sscc, string locationCode, DbTool dbTool, CultureInfo cultureDatabase)
        {

            string logUnitTableAlias = "LOGUNIT";

            DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();
            strBuilder.Append("SELECT ");

            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.ItemCode.NAME));

            strBuilder.Append(", ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.ItemName.NAME));

            strBuilder.Append(", ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTableKnighton.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTableKnighton.Columns.LiveLoadingType.NAME));

            strBuilder.Append(", ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxLiveLoadingDefinitionTableKnighton.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxLiveLoadingDefinitionTableKnighton.Columns.Is3PL.NAME));

            strBuilder.Append(", ");
            strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTable.NAME, PmxItemTable.Columns.DefaultStorageLocationOrZoneCode.NAME, PmxItemTable.Columns.DefaultStorageLocationOrZoneCode.NAME));


            strBuilder.Append(", ");
            strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTable.NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME));

            strBuilder.Append(", ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.StorLocCode.NAME));

            strBuilder.Append(", ");
            strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(logUnitTableAlias, PmxLogisticUnitIDTable.Columns.LogisticCarrierCode.NAME, PmxLogisticUnitIDTable.Columns.LogisticCarrierCode.NAME));

            strBuilder.Append(", ");
            strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME));

            strBuilder.Append(", ");
            strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME));

            strBuilder.Append(", ");
            strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME));

            strBuilder.Append(", ");
            strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME));

            strBuilder.Append(", ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.QualityStatusCode.NAME));

            strBuilder.AppendFormat(", " + strBuilder.IFNULL + "({0}, {1}) AS {2} ",
                SqlCommandHelper.EscapeTableColumnName(SboUomTable.NAME, SboUomTable.Columns.UomCode.NAME),
                SqlCommandHelper.EscapeTableColumnName(SboItemTable.NAME, SboItemTable.Columns.InventoryUom.NAME),
                SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.InventoryUom.NAME));


            strBuilder.Append(", ");
            strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTable.NAME, PmxItemTable.Columns.UOM2.NAME, PmxItemTable.Columns.UOM2.NAME));


            strBuilder.Append(", SUM( ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME);
            strBuilder.Append(" ) AS ");
            strBuilder.Append(strBuilder.AsIdentifier(PmxInventoryTotalTable.Columns.Quantity.NAME));

            strBuilder.Append(", SUM( ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QuantityUom2.NAME);
            strBuilder.Append(" ) AS ");
            strBuilder.Append(strBuilder.AsIdentifier(PmxInventoryTotalTable.Columns.QuantityUom2.NAME));

            strBuilder.Append(", ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME));

            strBuilder.Append(" FROM ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));

            strBuilder.Append(" INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);


            strBuilder.Append(" INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));
            strBuilder.Append(" ON ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.Columns.ItemCode.NAME));
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.Columns.ItemCode.NAME));

            strBuilder.Append(" LEFT JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxLiveLoadingDefinitionTableKnighton.NAME));
            strBuilder.Append(" ON ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTableKnighton.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTableKnighton.Columns.LiveLoadingType.NAME));
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxLiveLoadingDefinitionTableKnighton.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxLiveLoadingDefinitionTableKnighton.Columns.Code.NAME));

            strBuilder.Append(" LEFT JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(SboUomTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, SboItemTable.Columns.InventoryUoMEntry.NAME);
            strBuilder.Append(" AND ");
            SqlCommandHelper.AddTableColumnName(strBuilder, SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);
            strBuilder.Append(" <> -1 ");



            strBuilder.Append(" LEFT JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.InternalKey.NAME);

            strBuilder.Append(" LEFT JOIN ( ");
            {
                strBuilder.AppendFormat(" SELECT {0}, {1}, {2}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME, PmxLinkMasterLUIDTable.Columns.LUID.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.SSCC.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.LogisticCarrierCode.NAME)
                    );
                strBuilder.AppendFormat(" FROM {0}",
                    SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME)
                    );
                strBuilder.Append(" UNION ");
                strBuilder.AppendFormat(" SELECT DISTINCT {0}, {1}, {2}",
                     SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxLinkMasterLUIDTable.NAME, PmxLinkMasterLUIDTable.Columns.LUID.NAME, PmxLinkMasterLUIDTable.Columns.LUID.NAME),
                     SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.SSCC.NAME),
                     SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.LogisticCarrierCode.NAME)
                     );
                strBuilder.AppendFormat(" FROM {0}",
                    SqlCommandHelper.EscapeTableName(PmxLinkMasterLUIDTable.NAME)
                    );
                strBuilder.AppendFormat(" INNER JOIN {0} ON {1} = {2}",
                   SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME),
                     SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME),
                     SqlCommandHelper.EscapeTableColumnName(PmxLinkMasterLUIDTable.NAME, PmxLinkMasterLUIDTable.Columns.MasterLUID.NAME)
                   );
            }
            strBuilder.AppendFormat(" ) {0} ON {1} = {2}",
                logUnitTableAlias,
                SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME),
                SqlCommandHelper.EscapeTableColumnName(logUnitTableAlias, PmxLinkMasterLUIDTable.Columns.LUID.NAME)
                );

            strBuilder.Append(" WHERE ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(logUnitTableAlias));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxLogisticUnitIDTable.Columns.SSCC.NAME));
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(sscc, cultureDatabase));
            strBuilder.Append(" AND ( ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.IsLogisticCarrier.NAME));
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, cultureDatabase));
            strBuilder.Append(" OR ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxItemTable.Columns.IsLogisticCarrier.NAME));
            strBuilder.Append(" IS NULL ");
            strBuilder.Append(") ");

            if (!string.IsNullOrEmpty(locationCode))
            {
                strBuilder.Append(" AND ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.StorLocCode.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, cultureDatabase));
            }


            strBuilder.Append(" GROUP BY ");

            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, SboItemTable.Columns.ItemName.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.DefaultStorageLocationOrZoneCode.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QualityStatusCode.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, SboItemTable.Columns.InventoryUom.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, logUnitTableAlias, PmxLogisticUnitIDTable.Columns.LogisticCarrierCode.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, SboUomTable.NAME, SboUomTable.Columns.UomCode.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.UOM2.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxLiveLoadingDefinitionTableKnighton.NAME, PmxLiveLoadingDefinitionTableKnighton.Columns.Is3PL.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTableKnighton.NAME, PmxItemTableKnighton.Columns.LiveLoadingType.NAME);

            strBuilder.Append(" ORDER BY ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.ItemCode.NAME));
            return strBuilder.ToString();
        }


        /// <summary>
        /// Command for getting all products order by name
        /// </summary>
        /// <param name="sscc">The SSCC.</param>
        /// <param name="locationCode">The location code.</param>
        /// <param name="dbTool">The db tool.</param>
        /// <returns>
        /// Command for getting all products order by name
        /// </returns>
        public static string GetProductOnSSCCIncorrectQualityStatus(string sscc, string locationCode, DbTool dbTool, CultureInfo cultureDatabase)
        {
            DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();
            strBuilder.Append("SELECT ");

            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.ItemCode.NAME));


            strBuilder.Append(" FROM ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));



            strBuilder.Append(" INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxQualityStatusTable.NAME));
            strBuilder.Append(" ON ");
            strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxQualityStatusTable.NAME, PmxQualityStatusTable.Columns.Code.NAME));
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QualityStatusCode.NAME));

            strBuilder.Append(" WHERE ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.SSCC.NAME));
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(sscc, cultureDatabase));

            strBuilder.Append(" AND ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxQualityStatusTable.NAME));
            strBuilder.Append(".");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxQualityStatusTable.Columns.CanBeShipped.NAME));
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.N.VALUE, cultureDatabase));

            if (!string.IsNullOrEmpty(locationCode))
            {
                strBuilder.Append(" AND ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.StorLocCode.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, cultureDatabase));
            }


            return strBuilder.ToString();
        }



        internal static string GetOpenSalesOrderLinesWithoutLockedQuantity( string itemCode , bool justTheFirstLine , string cardCode , string shipToAddress , DbTool dbTool , CultureInfo cultureDatabase )
        {
            string subTable = "QuantityQuery";
            DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();

            strBuilder.Append( "SELECT " );
            if ( justTheFirstLine )
            {
                strBuilder.Append( " TOP (1) " );
            }
            strBuilder.Append( strBuilder.AsIdentifier( subTable , PmxSalesOrderTable.Columns.DocEntry.NAME ) ,
                ", " , strBuilder.AsIdentifier( subTable , PmxSalesOrderTable.Columns.DocNum.NAME ) ,
                ", " , strBuilder.AsIdentifier( subTable , PmxSalesOrderLineTable.Columns.LineNum.NAME ) ,
                ", " , strBuilder.AsIdentifier( subTable , SboSalesOrderTable.Columns.CardCode.NAME ) ,
                 ", " , strBuilder.AsIdentifier( subTable , SboSalesOrderTable.Columns.CardName.NAME ) ,
                ", " , strBuilder.AsIdentifier( subTable , SboSalesOrderLineTable.Columns.ShipToDesc.NAME ) ,
                ", (" , strBuilder.AsIdentifier( subTable , PmxSalesOrderLineTable.Columns.OpenInvQty.NAME ) ,
                " - " , strBuilder.AsIdentifier( SboSalesOrderLineTable.Columns.Quantity.NAME ) , ") AS " , strBuilder.AsIdentifier( SboSalesOrderLineTable.Columns.Quantity.NAME ) ,
                "\r\n FROM (" ,

                "SELECT " , strBuilder.AsIdentifier( PmxSalesOrderTable.NAME , PmxSalesOrderTable.Columns.DocEntry.NAME ) ,
                     ", " , strBuilder.AsIdentifier( PmxSalesOrderTable.NAME , PmxSalesOrderTable.Columns.DocNum.NAME ) ,
                     ", " , strBuilder.AsIdentifier( SboSalesOrderTable.NAME , SboSalesOrderTable.Columns.CardCode.NAME ) ,
                     ", " , strBuilder.AsIdentifier( SboSalesOrderTable.NAME , SboSalesOrderTable.Columns.CardName.NAME ) ,
                     ", " , strBuilder.AsIdentifier( SboSalesOrderLineTable.NAME , SboSalesOrderLineTable.Columns.ShipToDesc.NAME ) ,
                     ", " , strBuilder.AsIdentifier( PmxSalesOrderLineTable.NAME , PmxSalesOrderLineTable.Columns.OpenInvQty.NAME ) ,
                     ", " , strBuilder.AsIdentifier( PmxSalesOrderLineTable.NAME , PmxSalesOrderLineTable.Columns.LineNum.NAME ) ,
                     ", " , strBuilder.IFNULL , "( SUM( " , strBuilder.AsIdentifier( PmxInventoryLockingDetailTable.NAME , PmxInventoryLockingDetailTable.Columns.Quantity.NAME ) ,
                         " ), 0 ) AS " , strBuilder.AsIdentifier( SboSalesOrderLineTable.Columns.Quantity.NAME ) ,

                "\r\n FROM " , strBuilder.AsIdentifier( PmxSalesOrderLineTable.NAME ) ,

                 "\r\n INNER JOIN " , strBuilder.AsIdentifier( PmxSalesOrderTable.NAME ) ,
                           " ON " , strBuilder.AsIdentifier( PmxSalesOrderTable.NAME , PmxSalesOrderTable.Columns.DocEntry.NAME ) ,
                            " = " , strBuilder.AsIdentifier( PmxSalesOrderLineTable.NAME , PmxSalesOrderLineTable.Columns.DocEntry.NAME ) ,

                "\r\n LEFT JOIN " , strBuilder.AsIdentifier( PmxInventoryLockingDetailTable.NAME ) ,
                           " ON " , strBuilder.AsIdentifier( PmxInventoryLockingDetailTable.NAME , PmxInventoryLockingDetailTable.Columns.BaseEntry.NAME ) ,
                            " = " , strBuilder.AsIdentifier( PmxSalesOrderLineTable.NAME , PmxSalesOrderLineTable.Columns.DocEntry.NAME ) ,
                          " AND " , strBuilder.AsIdentifier( PmxInventoryLockingDetailTable.NAME , PmxInventoryLockingDetailTable.Columns.BaseLine.NAME ) ,
                            " = " , strBuilder.AsIdentifier( PmxSalesOrderLineTable.NAME , PmxSalesOrderLineTable.Columns.LineNum.NAME ) ,
                          " AND " , strBuilder.AsIdentifier( PmxInventoryLockingDetailTable.NAME , PmxInventoryLockingDetailTable.Columns.BaseType.NAME ) ,
                            " = " , strBuilder.AsIdentifier( PmxSalesOrderLineTable.NAME , SboSalesOrderLineTable.Columns.ObjectType.NAME ) ,

                "\r\n WHERE " , strBuilder.AsIdentifier( PmxSalesOrderLineTable.NAME , SboSalesOrderLineTable.Columns.ItemCode.NAME ) ,
                        " = " , strBuilder.AsString( itemCode ) ,
                      " AND " , strBuilder.AsIdentifier( PmxSalesOrderLineTable.NAME , SboSalesOrderLineTable.Columns.LineStatus.NAME ) ,
                        " = " , SqlCommandHelper.FormatAndEscapeColumnValue( SboDocStatusColumnValues.Open.VALUE , cultureDatabase ) );
            if (!justTheFirstLine &&
				!string.IsNullOrEmpty(shipToAddress))
            {
                strBuilder.Append( " AND " , strBuilder.AsIdentifier( SboSalesOrderLineTable.NAME , SboSalesOrderLineTable.Columns.ShipToDesc.NAME ) ,
                       " = " , SqlCommandHelper.FormatAndEscapeColumnValue( shipToAddress , cultureDatabase ) );
            }

            strBuilder.Append( "\r\n GROUP BY " , strBuilder.AsIdentifier( PmxSalesOrderTable.NAME , PmxSalesOrderTable.Columns.DocEntry.NAME ) ,
                     ", " , strBuilder.AsIdentifier( PmxSalesOrderTable.NAME , PmxSalesOrderTable.Columns.DocNum.NAME ) ,
                     ", " , strBuilder.AsIdentifier( PmxSalesOrderTable.NAME , SboSalesOrderTable.Columns.CardCode.NAME ) ,
                     ", " , strBuilder.AsIdentifier( PmxSalesOrderTable.NAME , SboSalesOrderTable.Columns.CardName.NAME ) ,
                     ", " , strBuilder.AsIdentifier(  SboSalesOrderLineTable.NAME , SboSalesOrderLineTable.Columns.ShipToDesc.NAME ) ,
                     ", " , strBuilder.AsIdentifier( PmxSalesOrderLineTable.NAME , PmxSalesOrderLineTable.Columns.OpenInvQty.NAME ) ,
                     ", " , strBuilder.AsIdentifier( PmxSalesOrderLineTable.NAME , PmxSalesOrderLineTable.Columns.LineNum.NAME ) ,

                "\r\n ) AS  " , strBuilder.AsIdentifier( subTable ) ,
                "\r\n WHERE (" , strBuilder.AsIdentifier( subTable , SboSalesOrderLineTable.Columns.OpenInvQty.NAME ) ,
                " - " , strBuilder.AsIdentifier( subTable , SboSalesOrderLineTable.Columns.Quantity.NAME ) , ") > 0 " ,

                "\r\n ORDER BY " , strBuilder.AsIdentifier( subTable , PmxSalesOrderTable.Columns.DocEntry.NAME ) ,
                ", " , strBuilder.AsIdentifier( subTable , PmxSalesOrderLineTable.Columns.LineNum.NAME ) );

            return strBuilder.ToString();
        }

        /// <summary>
        /// Gets all loading docks of a warehouse for live loading.
        /// </summary>
        /// <param name="oseWarehouseCode">The ose warehouse code.</param>
        /// <param name="liveLoadingCode">The live loading code.</param>
        /// <param name="cultureDatabase">The culture database.</param>
        /// <returns></returns>
        internal static string GetAllLoadingDocksForTransfer(string locationCode, string oseWarehouseCode, CultureInfo cultureDatabase)
        {
            StringBuilder strBuilder = new StringBuilder();
            strBuilder.Append("SELECT DISTINCT ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Code.NAME, PmxOseDockWithWarehouseViewDefinition.Columns.Code.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Name.NAME, LocationDataSet.Columns.DESCRIPTION.Name);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, LocationDataSet.Columns.WAREHOUSECODE.Name);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, LocationDataSet.Columns.WAREHOUSENAME.Name);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxLiveLoadingDefinitionTableKnighton.NAME, PmxLiveLoadingDefinitionTableKnighton.Columns.Is3PL.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxLiveLoadingDefinitionTableKnighton.NAME, PmxLiveLoadingDefinitionTableKnighton.Columns.SboWarehouseCode.NAME);
            strBuilder.Append(", ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxLiveLoadingDefinitionTableKnighton.NAME, PmxLiveLoadingDefinitionTableKnighton.Columns.DockCode.NAME);
            strBuilder.Append(" FROM ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
            strBuilder.Append(" INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseDockTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Code.NAME);
            strBuilder.Append(" INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Code.NAME);
            strBuilder.Append(" INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME);

            strBuilder.Append(" INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);

            strBuilder.Append(" INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, SboItemTable.Columns.ItemCode.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME);

            strBuilder.Append(" INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxLiveLoadingDefinitionTableKnighton.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxLiveLoadingDefinitionTableKnighton.NAME, PmxLiveLoadingDefinitionTableKnighton.Columns.Code.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTableKnighton.NAME, PmxItemTableKnighton.Columns.LiveLoadingType.NAME);


            strBuilder.Append(" WHERE ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseDockTable.Columns.IsLoadingDock.NAME));
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, cultureDatabase));
            strBuilder.Append(" AND ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.IsActive.NAME);
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, cultureDatabase));

            if (!string.IsNullOrWhiteSpace(locationCode))
            {
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Code.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, cultureDatabase));


            }


            if (!string.IsNullOrEmpty(oseWarehouseCode))
            {
                strBuilder.Append(" AND ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(oseWarehouseCode, cultureDatabase));
            }

            strBuilder.Append(" ORDER BY ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Name.NAME);

            return strBuilder.ToString();
        }

        internal static string GetLogisticUnitCountOnLocation(string locationCode, string oseWarehouseCode, CultureInfo cultureDatabase)
        {
            StringBuilder strBuilder = new StringBuilder();
            strBuilder.Append("SELECT COUNT(DISTINCT ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME);
            strBuilder.Append(") ");
            strBuilder.Append("\n FROM ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
            strBuilder.Append("\n INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseDockTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Code.NAME);
            strBuilder.Append("\n INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Code.NAME);
            strBuilder.Append("\n INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME);


            strBuilder.Append("\n INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);


            strBuilder.Append("\n WHERE ");

            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Code.NAME);
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, cultureDatabase));
            strBuilder.Append(" AND ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME));
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(oseWarehouseCode, cultureDatabase));
            strBuilder.Append(" AND ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME);
            strBuilder.Append(" IS NOT NULL ");

            //strBuilder.Append("\n GROUP BY ");
            //SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME);

            return strBuilder.ToString();
        }


        internal static string GetLuidsOnLocation(string locationCode, string oseWarehouseCode, CultureInfo cultureDatabase)
        {
            StringBuilder strBuilder = new StringBuilder();
            strBuilder.Append("SELECT DISTINCT ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME);
            strBuilder.Append("\n FROM ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
            strBuilder.Append("\n INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseDockTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Code.NAME);
            strBuilder.Append("\n INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Code.NAME);
            strBuilder.Append("\n INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME);


            strBuilder.Append("\n LEFT JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);


            strBuilder.Append(" WHERE \n ");

            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseDockTable.NAME, PmxOseDockTable.Columns.Code.NAME);
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, cultureDatabase));
            strBuilder.Append("\n AND ");
            strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME));
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(oseWarehouseCode, cultureDatabase));
            strBuilder.Append(" AND ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME);
            strBuilder.Append(" IS NOT NULL ");

            return strBuilder.ToString();
        }


        internal static string GetLockedItemsForLocation(string locationCode, CultureInfo cultureDatabase)
        {
            StringBuilder strBuilder = new StringBuilder();
            strBuilder.Append("SELECT  ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.StorLocCode.NAME);
            strBuilder.Append(",  ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.BaseEntry.NAME);
            strBuilder.Append(",  ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.BaseLine.NAME);
            strBuilder.Append(",  ");
            SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, SboItemTable.Columns.ItemCode.NAME);
            strBuilder.Append(",  ");
            SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, SboItemTable.Columns.ItemName.NAME);
            strBuilder.Append(",  ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.LogUnitIdentKey.NAME);
            strBuilder.Append(",  ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.SSCC.NAME);
            strBuilder.Append(",  ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.QualityStatusCode.NAME);
            strBuilder.Append(",  ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.SboWarehouseCode.NAME);
            strBuilder.Append(",  ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME);
            strBuilder.Append(",  ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber2.NAME);
            strBuilder.Append(",  ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME);
            strBuilder.Append(",  ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.Quantity.NAME);
            strBuilder.Append(",  ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.UOM.NAME);
            strBuilder.Append(",  ");
            SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, PmxItemTable.Columns.NumberOfItemsPerSalesUnit.NAME);
            strBuilder.Append(",  ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.QuantityUom2.NAME);
            strBuilder.Append(",  ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.UOM2.NAME);
            strBuilder.Append(",  ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.IsCatchWeightItem.NAME);
            strBuilder.Append(",  ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.CardCode.NAME);
            strBuilder.Append(",  ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.CardName.NAME);

            strBuilder.Append("\n FROM ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryLockingDetailTable.NAME));

            strBuilder.Append("\n INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.StorLocCode.NAME);

            strBuilder.Append("\n INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.PmxWhsCode.NAME);

            strBuilder.Append("\n INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.ItemCode.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.ItemCode.NAME);

            strBuilder.Append("\n INNER JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.LogUnitIdentKey.NAME);

            strBuilder.Append("\n LEFT JOIN ");
            strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
            strBuilder.Append(" ON ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.InternalKey.NAME);
            strBuilder.Append(" = ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.ItemTransactionalInfoKey.NAME);


            strBuilder.Append("\n WHERE ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.StorLocCode.NAME);
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, cultureDatabase));

            strBuilder.Append("\n AND ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.BaseType.NAME);
            strBuilder.Append(" = ");
            strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboObjectTypes.SalesOrder, cultureDatabase));

            strBuilder.Append("\n ORDER BY ");
            SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.BaseEntry.NAME);

            return strBuilder.ToString();


        }

        internal static string GetRemoveInventoryLockingByLuidList(List<int> luidList, CultureInfo cultureDatabase)
        {
            StringBuilder query = new StringBuilder();

            query.AppendFormat("DELETE FROM {0}",
                SqlCommandHelper.EscapeTableName(PmxInventoryLockingDetailTable.NAME));
            query.Append("\r\nWHERE ");
            query.AppendFormat(" {0} IN ( ", SqlCommandHelper.EscapeTableColumnName(PmxInventoryLockingDetailTable.NAME, PmxInventoryLockingDetailTable.Columns.LogUnitIdentKey.NAME));
            query.Append(SqlCommandHelper.FormatAndEscapeColumnValue(luidList, cultureDatabase));
            query.Append(") ");

            return query.ToString();
        }




        //            select* from PMX_INLD
        //inner join pmx_osel on PMX_OSEL.Code = pmx_inld.StorLocCode
        //inner join oitm on oitm.ItemCode = PMX_INLD.ItemCode
        //left join PMX_LUID on PMX_LUID.InternalKey = PMX_INLD.LogUnitIdentKey
        //left join pmx_itri on pmx_itri.InternalKey = pmx_inld.ItemTransactionalInfoKey

    }

    #endregion
}
