﻿using Produmex.Foundation;
using Produmex.Foundation.Convertors;
using Produmex.Foundation.Data;
using Produmex.Foundation.Data.DbClient;
using Produmex.Foundation.Data.Sbo;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions;
using Produmex.Foundation.Data.Sbo.BusinessObjects.Definitions.Tables;
using Produmex.Foundation.Data.Sbo.DataObjects;
using Produmex.Foundation.Data.Sbo.Generators.BarcodeController;
using Produmex.Foundation.Data.Sbo.Utilities;
using Produmex.Foundation.Data.SqlClient;
using Produmex.Foundation.Diagnostics;
using Produmex.Foundation.Messages;
using Produmex.Foundation.SlimScreen;
using Produmex.Foundation.SlimScreen.Interfaces;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions;
using Produmex.Foundation.SlimScreen.Interfaces.Definitions.KnownDataSets;
using Produmex.Foundation.Utilities;
using Produmex.Foundation.Workflows;
using Produmex.Foundation.Workflows.Parameters;
using Produmex.Foundation.Wwf.Sbo.LocalServices;
using Produmex.Sbo.Logex.Data;
using Produmex.Sbo.Logex.Data.BusinessObjects;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.Tables;
using Produmex.Sbo.Logex.Data.DataObjects;
using Produmex.Sbo.Logex.Data.Providers;
using Produmex.Sbo.Logex.Data.ViewObjects;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Data;
using System.Globalization;
using System.Linq;
using System.Reflection;
using System.Text;
using System.Threading.Tasks;
using System.Transactions;
using Produmex.Sbo.Logex.Data.BusinessObjects.Definitions;

// PmxWorkflowExecutionTypes.SUB_FLOW
namespace WorkflowScript_MoveSSCCForProduction
{
    public class WorkflowScript_MoveSSCCForProductionScript : WorkflowInstanceScriptBase
    {
        public WorkflowScript_MoveSSCCForProductionScript(WorkflowInstanceBase parent, WorkflowInstanceFactory factory) : base(parent, factory)
        {
            m_afterSSCCLoadedHookFlow = this.Create("AfterSSCCLoadedHookScript");
        }

        // Input parameters
        public ReadOnlyBinder<CultureInfo> DefaultCultureInfo;
        public ReadOnlyBinder<PmxOseCompany> PmxOseCompany;

        public ReadOnlyBinder<string> SrcLocationCode;
        public ReadOnlyBinder<string> DstLocationCode;

        WorkflowInstanceBase m_afterSSCCLoadedHookFlow;

        protected string m_subFlowWarehouseTransferDestinationPmxWhsCode;

        private static readonly ILog s_log = LogProvider.GetLogger(MethodInfo.GetCurrentMethod().DeclaringType);

        ISboProviderService m_sboProviderService;

        #region WorkflowInstanceScriptBase Members

        protected override void Execute()
        {
            Session session = GetScopeParameter("Session") as Session;
            ISboProviderService sboProviderService = GetScopeParameter("<WwfService>ISboService") as ISboProviderService;
            m_sboProviderService = (ISboProviderService)GetScopeParameter("<WwfService>ISboService");

            string initialErrorKey = null;
            Message msg;
            ProdumexError pmxError = null;
            IBarcodeController barcodeController = null;
            string SSCC = null;
            int? LUID = null;
            int numberOfReports = 0;
            bool goBack = false;
            bool askReprint = false;
            string query = null;
            string dstLocationCode = null;
            string warehouseCode = null;
            PmxOseWarehouseInfo whsInfo = null;
            DataSet ds = null;
            DataSet dsDestinationLocations = null;

        #region Scan SSCC
        Step_ScanSSCC:

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.Reception.IIdentifySSCCScreen),
                    this.DefaultCultureInfo, BuildParamCollection(
                    "InitialErrorKey", initialErrorKey,
                    "TitleKey", "MSG_TITLE_IDENTIFY_SSCC",
                    "BarcodeController", barcodeController,
                    "ButtonNoSSCC", false));
            initialErrorKey = null;
            msg = WaitForMessage();

            if (msg.Name.EndsWith(".SSCCEntered"))
            {
                SSCC = ExtractParameter<string>(msg.Parameters, "sscc");
                LUID = m_sboProviderService.InvokeMethodWithDbConnection<int?>(false, false, null, null,
                    delegate (PmxDbConnection conn, object[] parameters)
                    {
                        PmxLogisticUnitIDProvider luidProv = new PmxLogisticUnitIDProvider(conn);
                        return luidProv.GetLUIDBySSCC(SSCC, true);
                    });

                if (LUID <= 0)
                {
                    initialErrorKey = "MSG_ERROR_INVALID_SSCC";
                    goto Step_ScanSSCC;
                }

                goto Step_ScanLocation;
            }
            if (msg.Name.EndsWith(".BackRequested"))
            {
                return;
            }

            goto Step_ScanSSCC;

        #endregion

        #region Scan or select location
        Step_ScanLocation:


            string inputLocationCode = null;
            

            //Get the destination locations of the LUID
            query = BuildQuery.GetDestinationLocationByLUID(LUID, this.DefaultCultureInfo);
            ds = sboProviderService.RunView(false, null, null, query);

            if (GetNumberOfRows(ds) > 1)
            {
                //Multiple rows, so take the input location
                inputLocationCode = ds.Tables[0].Rows[0][PmxPickListTable.Columns.MoveToLocationCode.NAME] as string;
            }
            else if (GetNumberOfRows(ds) == 1)
            {
                inputLocationCode = ds.Tables[0].Rows[0][PmxLinedUpLocationTable.Columns.LinedUpLocationCode.NAME] as string;

                if (string.IsNullOrEmpty(inputLocationCode))
                {
                    //Lined up location is empty, so use the input location
                    inputLocationCode = ds.Tables[0].Rows[0][PmxPickListTable.Columns.MoveToLocationCode.NAME] as string;
                }
            }
            else
            {
                //No Production order found, so SSCC is not correct.
                initialErrorKey = "MSG_ERROR_INVALID_SSCC";
                goto Step_ScanSSCC;
            }


            int docEntry = Convert.ToInt32(ds.Tables[0].Rows[0][PmxPickListTable.Columns.DocEntry.NAME]);

            goto Step_SelectLocation;

            //Collection<PmxLinedUpLocation> linedUpLocations = null;

        //query = BuildQuery.GetDestinationLocationByLUID(LUID, this.DefaultCultureInfo);
        //ds = sboProviderService.RunView(false, null, null, query);
        //DataRow[] dr = ds.Tables[0].Select(this.DstLocationCode);
        //string inputLocationCode = Convert.ToString(dr[0][PmxPickListTable.Columns.MoveToLocationCode.NAME]);
        //int waveKey = Convert.ToInt32(dr[0][PmxPickListTable.Columns.WaveKey.NAME]);
        //int docEntry = Convert.ToInt32(dr[0][PmxPickListTable.Columns.DocEntry.NAME]);

        //query = BuildQuery.GetProductionLine(docEntry, this.DefaultCultureInfo);
        //ds = sboProviderService.RunView(false, null, null, query);
        //dr = ds.Tables[0].Select();
        //string productionLineCode = Convert.ToString(dr[0][PmxProductionOrderTable.Columns.ProductionLineCode.NAME]);
        //int productionOrder = Convert.ToInt32(dr[0][PmxProductionOrderTable.Columns.DocEntry.NAME]);
        //int productionLineNum = Convert.ToInt32(dr[0][PmxProductionOrderLineTable.Columns.LineNum.NAME]);
        //bool hasToBeLinedUp = dr[0][PmxProductionOrderLineTable.Columns.HasToBeLinedUp.NAME].ToString() == "Y" ? true : false;

        //for (int i = 1; i < dr.Length; i++)
        //{
        //    if (dr[i][PmxProductionOrderLineTable.Columns.HasToBeLinedUp.NAME].ToString() == "Y" && !hasToBeLinedUp ||
        //        dr[i][PmxProductionOrderLineTable.Columns.HasToBeLinedUp.NAME].ToString() == "N" && hasToBeLinedUp)
        //    {
        //        hasToBeLinedUp = false;
        //        break;
        //    }
        //}

        //Collection<KeyValuePair<string, int>> listLinedUpLocations = null;

        //if (hasToBeLinedUp)
        //{
        //    m_sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null,
        //                 delegate (PmxDbConnection conn, object[] parameters)
        //                 {
        //                     PmxLinedUpLocationProvider lupprov = new PmxLinedUpLocationProvider(conn);
        //                     linedUpLocations = lupprov.GetLinedUpLocationsForProductionLine(productionLineCode, null, productionOrder);
        //                     return null;
        //                 });

        //    if (linedUpLocations.Count > 0)
        //    {
        //        //there is at least one lined up location assigned to the production line
        //        inputLocationCode = linedUpLocations[0].LinedUpLocationCode;
        //        goto Step_SelectLocation;
        //    }
        //    else
        //    {
        //        //there is no lined up location assigned to the production line
        //        whsInfo = sboProviderService.InvokeMethodWithDbConnection<PmxOseWarehouseInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOseWarehouseInfo>(ExecuteGetPmxWarehouseCode), inputLocationCode);
        //        query = BuildQuery.GetLinedUpLocationForProductionLine(productionLineCode, whsInfo.PmxWarehouseCode, whsInfo.WarehouseName, this.DefaultCultureInfo);
        //        ds = sboProviderService.RunView(false, null, null, query);
        //        //ds.Tables[0].Columns.Add(LocationDataSet.Columns.CODE.Name, (Type)string, 

        //        if (GetRowCount(ds) > 0)
        //        {
        //            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectLocationScreen),
        //                                DefaultCultureInfo.Get(), BuildParamCollection(
        //                                "InitialErrorKey", initialErrorKey,
        //                                "TitleKey", "MSG_TITLE_SELECT_LINE_UP_LOCATION",
        //                                "LocationDS", ds,
        //                                "ShowStopButton", false));
        //            initialErrorKey = null;
        //            msg = WaitForMessage();


        //            if (msg.Name.EndsWith(".BackRequested"))
        //            {
        //                goBack = true;
        //                goto Step_ScanSSCC;
        //            }
        //            if (msg.Name.EndsWith(".LocationSelected") || msg.Name.EndsWith(".ForwardRequested"))
        //            {
        //                dstLocationCode = ExtractParameter<string>(msg.Parameters, "locationCode");

        //                if (!string.IsNullOrEmpty(dstLocationCode))
        //                {
        //                    listLinedUpLocations = new Collection<KeyValuePair<string, int>>();
        //                    listLinedUpLocations.Add(new KeyValuePair<string, int>(dstLocationCode, productionLineNum));

        //                    m_sboProviderService.InvokeMethodWithDbConnection<object>(true, false, null, null,
        //                                         delegate (PmxDbConnection conn, object[] parameters)
        //                                         {
        //                                             PmxLinedUpLocationProvider lupprov = new PmxLinedUpLocationProvider(conn);
        //                                             lupprov.AddLinedUpLocations(productionLineCode, productionOrder, listLinedUpLocations);
        //                                             return null;
        //                                         });

        //                    goto Step_SetSelectedLocation;
        //                }
        //                else
        //                {
        //                    initialErrorKey = "MSG_KF_ERROR_NO_LINED_UP_LOCATION_HAS_BEEN_SELECTED";
        //                }
        //            }
        //        }
        //        else
        //        {
        //            initialErrorKey = "MSG_ERROR_NO_POSSIBLE_LINE_UP_LOCATION";
        //        }
        //    }
        //    goto Step_ScanLocation;
        //}

        #region SELECT LOCATION
        Step_SelectLocation:

            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.MoveOrder.IIdentifyLocationScreen),
                            DefaultCultureInfo.Get(), BuildParamCollection(
                                "InitialErrorKey", initialErrorKey,
                                "TitleKey", "MSG_TITLE_SCAN_DESTINATION_LOCATION",
                                "DefaultLocation", inputLocationCode,
                                "AllowedButtons", new string[] { "MSG_BUTTON_SELECT_LOCATION" }
                                ));

            initialErrorKey = null;
            msg = WaitForMessage();

            if (msg.Name.EndsWith(".ButtonClicked"))
            {
                whsInfo = sboProviderService.InvokeMethodWithDbConnection<PmxOseWarehouseInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOseWarehouseInfo>(ExecuteGetPmxWarehouseCode), inputLocationCode);

                warehouseCode = whsInfo.PmxWarehouseCode;

                query = BuildQuery.GetStorageLocations(warehouseCode, sboProviderService.GetDbTool());

                dsDestinationLocations = sboProviderService.RunView(false, null, null, query);

                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.ISelectLocationScreen),
                                        DefaultCultureInfo.Get(), BuildParamCollection(
                                        "InitialErrorKey", initialErrorKey,
                                        "TitleKey", "MSG_TITLE_SELECT_DESTINATION_LOCATION",
                                        "LocationDS", dsDestinationLocations,
                                        "ShowStopButton", false));
                initialErrorKey = null;
                msg = WaitForMessage();
            }

            if (msg.Name.EndsWith(".LocationEntered") || msg.Name.EndsWith(".LocationSelected") || msg.Name.EndsWith(".ForwardRequested"))
            {
                goto Step_SetSelectedLocation;
            }

            if (msg.Name.EndsWith(".BackRequested"))
            {
                goBack = true;
                goto Step_ScanSSCC;
            }

            goto Step_ScanLocation;

        #endregion

        #endregion

        #region Set selected location
        Step_SetSelectedLocation:

            dstLocationCode = ExtractParameter<string>(msg.Parameters, "locationCode");

            if (dstLocationCode.ToUpper() == inputLocationCode.ToUpper())
            {
                //Scanned the same
                if (sboProviderService.InvokeMethodWithDbConnection<bool>(false, false, null, null, new DelegateWithPmxDbConnection<bool>(ExecuteIsLocationLocked), dstLocationCode))
                {
                    //Location is locked
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", "MSG_ERROR_DESTINATION_LOCATION_LOCKED",
                            "NoTranslationOfMessageKey", false,
                            "ShowButton", true));
                    initialErrorKey = null;
                    WaitForMessage();

                    goto Step_ScanLocation;
                }

                goto Step_LoadSSCC;
            }
            //else scan another location
            session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                    DefaultCultureInfo.Get(), BuildParamCollection(
                        "TitleKey", "MSG_KF_TITLE_CONFIRM_OTHER_INPUT_LOCATION",
                        "MessageKey", "MSG_KF_QUESTION_CONFIRM_OTHER_INPUT_LOCATION"));
            msg = WaitForMessage();

            if (msg.Name.EndsWith("IDecisionScreenReply.Yes"))
            {
                inputLocationCode = dstLocationCode;
                goto Step_LoadSSCC;
            }

            //initialErrorKey = "MSG_KF_ERROR_INVALID_LOCATION";
            goto Step_ScanLocation;

        #endregion

        #region Load the SSCC
        Step_LoadSSCC:

            query = BuildQuery.GetProductOnSSCC(SSCC, null, sboProviderService.GetDbTool());
            ds = sboProviderService.RunView(false, null, null, query);

            if (GetNumberOfRows(ds) == 0)
            {
                initialErrorKey = "MSG_ERROR_INVALID_SSCC";
                goto Step_ScanSSCC;
            }

            //get warehouse
            whsInfo = sboProviderService.InvokeMethodWithDbConnection<PmxOseWarehouseInfo>(false, false, null, null, new DelegateWithPmxDbConnection<PmxOseWarehouseInfo>(ExecuteGetPmxWarehouseCode), ds.Tables[0].Rows[0][PmxInventoryTotalTable.Columns.StorLocCode.NAME] as string);

            goto Step_WarehouseTransfer;

        #endregion

        #region Warehouse Transfer
        Step_WarehouseTransfer:

            try
            {
                sboProviderService.InvokeMethodWithDbConnection<object>(false, false, null, null, new DelegateWithPmxDbConnection<object>(ExecuteLoadLogisticUnit), LUID);
            }
            catch (Exception ex)
            {
                s_log.Error("", ex);
                if (!(ex is ProdumexException))
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "MessageKey", ExceptionHelper.Format(ExceptionHelper.Content.InnerExceptions, ex, DefaultCultureInfo.Get()),
                            "NoTranslationOfMessageKey", true,
                            "ShowButton", true));
                    WaitForMessage();
                }
                else
                {
                    // Inform the user with the error
                    session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IShowMessageScreen),
                        DefaultCultureInfo.Get(), BuildParamCollection(
                            "PmxError", ((ProdumexException)ex).Error,
                            "ShowButton", true));
                    WaitForMessage();
                }

                session.ShowScreen(typeof(Produmex.Foundation.SlimScreen.Interfaces.IDecisionScreen),
                DefaultCultureInfo.Get(), BuildParamCollection(
                "TitleKey", "MSG_TITLE_ERROR_TRY_AGAIN",
                "MessageKey", "MSG_QUESTION_ERROR_TRY_AGAIN"));
                msg = WaitForMessage();

                if (msg.Name.EndsWith(".Yes"))
                {
                    goto Step_WarehouseTransfer;
                }
                goto Step_ScanSSCC;
            }

            DocumentRef[] list = m_sboProviderService.InvokeMethodWithDbConnection<DocumentRef[]>(false, null, null, delegate (SboConnection conn, object[] parameters)
            {
                PmxStockTransferProvider prov = new PmxStockTransferProvider(conn);
                return prov.AddStockTransferForShipping(null, null, new Collection<int> { docEntry }, null, null, null, inputLocationCode);
            });

            goto Step_ScanSSCC;
        }

        #endregion
    
        #endregion

        #region HELPERS
        private static Collection<DocumentRef> ExecuteMakeAdHocMove(SboConnection conn, object[] parameters)
        {
            PmxMoveProvider prov = new PmxMoveProvider(conn);
            return prov.MakeAdHocMoveForLUIDs((IList<int>)parameters[0], (string)parameters[1], null, null, null);
        }

        private static bool ExecuteIsLocationLocked(PmxDbConnection conn, object[] parameters)
        {
            PmxOseStorageLocationProvider prov = new PmxOseStorageLocationProvider(conn);
            return prov.IsLocationLocked((string)parameters[0]);
        }

        private static int GetNumberOfRows(DataSet ds)
        {
            if (ds != null && ds.Tables.Count > 0)
            {
                return ds.Tables[0].Rows.Count;
            }
            return 0;
        }

        private static object ExecuteLoadLogisticUnit(PmxDbConnection conn, object[] parameters)
        {
            PmxLogisticUnitIDProvider prov = new PmxLogisticUnitIDProvider(conn);
            prov.LoadLogisticUnit((int)parameters[0]);
            return null;
        }

        private static PmxOseWarehouseInfo ExecuteGetPmxWarehouseCode(PmxDbConnection conn, object[] parameters)
        {
            PmxOseProvider oseProv = new PmxOseProvider(conn);
            return oseProv.GetPmxWarehouseInfo((string)parameters[0]);
        }
        #endregion

        #region BUILD QUERY
        private class BuildQuery
        {
            internal static string GetSSCCInfo(string sscc)
            {
                StringBuilder strBuilder = new StringBuilder();

                strBuilder.AppendFormat("SELECT DISTINCT {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME, SSCCDataSet.Columns.LUID.Name));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.SSCC.NAME, SSCCDataSet.Columns.SSCC.Name));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME, SSCCDataSet.Columns.STORAGE_LOCATION_CODE.Name));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, SSCCDataSet.Columns.PMX_WAREHOUSE_CODE.Name));
                strBuilder.AppendFormat(", {0}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, SSCCDataSet.Columns.PMX_WAREHOUSE_NAME.Name));

                strBuilder.AppendFormat("\r\n FROM {0}",
                    SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));

                strBuilder.AppendFormat("\r\n INNER JOIN {0}",
                    SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME));

                strBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME));

                strBuilder.AppendFormat("\r\n INNER JOIN {0}",
                SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));

                strBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME));

                strBuilder.AppendFormat("\r\n INNER JOIN {0}",
                    SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
                strBuilder.AppendFormat(" ON {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME));

                strBuilder.AppendFormat("\r\n WHERE {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.SSCC.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(sscc, CultureInfo.InvariantCulture));

                return strBuilder.ToString();
            }

            internal static string GetZoneTypesForItem(string itemCode)
            {
                StringBuilder strBuilder = new StringBuilder();

                strBuilder.AppendFormat("SELECT DISTINCT {0}",
                    SqlCommandHelper.EscapeTableColumnName(PmxItemZoneTypeTable.NAME, PmxItemZoneTypeTable.Columns.ZoneTypeCode.NAME));

                strBuilder.AppendFormat(" FROM {0}",
                    SqlCommandHelper.EscapeTableName(PmxItemZoneTypeTable.NAME));

                strBuilder.AppendFormat(" WHERE {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxItemZoneTypeTable.NAME, PmxItemZoneTypeTable.Columns.ItemCode.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(itemCode, CultureInfo.InvariantCulture));

                return strBuilder.ToString();
            }

            public static string GetStorageLocations(string warehouseCode, DbTool dbTool)
            {
                DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();
                strBuilder.Append("SELECT ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME, LocationDataSet.Columns.CODE.Name);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Name.NAME, LocationDataSet.Columns.DESCRIPTION.Name);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME, LocationDataSet.Columns.WAREHOUSECODE.Name);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Name.NAME, LocationDataSet.Columns.WAREHOUSENAME.Name);

                strBuilder.Append("\r\n FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.Append("\r\n INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseGeneralTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.Code.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseStorageLocationGeneralTable.NAME, PmxOseStorageLocationGeneralTable.Columns.Code.NAME);

                strBuilder.Append("\r\n INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxOseWarehouseTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseGeneralTable.NAME, PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.Code.NAME);

                strBuilder.Append("\r\n INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxWarehouseTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxWarehouseTable.NAME, PmxWarehouseTable.Columns.WarehouseCode.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxOseWarehouseTable.NAME, PmxOseWarehouseTable.Columns.SboWarehouseCode.NAME);

                strBuilder.Append("\r\n WHERE ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseStorageLocationGeneralTable.Columns.IsActive.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(SboBooleanColumnValues.Y.VALUE, CultureInfo.InvariantCulture));

                strBuilder.Append(" AND ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxOseGeneralTable.Columns.PmxWarehouseCode.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(warehouseCode, CultureInfo.InvariantCulture));

                return strBuilder.ToString();
            }

            public static string GetDestinationLocationByLUID(int? luid, CultureInfo cultureDatabase)
            {
               

                StringBuilder strBuilder = new StringBuilder();

                strBuilder.AppendFormat("SELECT DISTINCT {0}, {1}, {2} FROM {3}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxPickListTable.NAME, PmxPickListTable.Columns.MoveToLocationCode.NAME, PmxPickListTable.Columns.MoveToLocationCode.NAME),
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxPickListTable.NAME, PmxPickListTable.Columns.DocEntry.NAME, PmxPickListTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxLinedUpLocationTable.NAME, PmxLinedUpLocationTable.Columns.LinedUpLocationCode.NAME, PmxLinedUpLocationTable.Columns.LinedUpLocationCode.NAME),
                    SqlCommandHelper.EscapeTableName(PmxPickListTable.NAME));


                strBuilder.AppendFormat("\r\n INNER JOIN {0} ON {1} = {2}",
                    SqlCommandHelper.EscapeTableName(PmxPickListLineTable.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxPickListTable.NAME, PmxPickListTable.Columns.DocEntry.NAME));

                strBuilder.AppendFormat("\r\n INNER JOIN {0} ON {1} = {2} AND {3} = {4}",
                    SqlCommandHelper.EscapeTableName(PmxLinkPickListLastMoveTable.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.PickListDocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.PickListLineNum.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.LineNum.NAME));


                strBuilder.AppendFormat("\r\n INNER JOIN {0} ON {1} = {2} AND {3} = {4}",
                    SqlCommandHelper.EscapeTableName(PmxMoveLineTable.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.MoveDocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxLinkPickListLastMoveTable.NAME, PmxLinkPickListLastMoveTable.Columns.MoveLineNum.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.LineNum.NAME));


                strBuilder.AppendFormat("\r\n INNER JOIN {0} ON {1} = {2} AND {3} = {4} AND {5} = {6}",
                  SqlCommandHelper.EscapeTableName(PmxPickListProposalLineTable.NAME),
                  SqlCommandHelper.EscapeTableColumnName(PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.BaseEntry.NAME),
                  SqlCommandHelper.EscapeTableColumnName(PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.DocEntry.NAME),
                  SqlCommandHelper.EscapeTableColumnName(PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.BaseLine.NAME),
                  SqlCommandHelper.EscapeTableColumnName(PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.LineNum.NAME),
                  SqlCommandHelper.EscapeTableColumnName(PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.BaseType.NAME),
                  SqlCommandHelper.FormatAndEscapeColumnValue(Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.PmxObjectTypes.PmxPickListProposal, cultureDatabase));

                strBuilder.AppendFormat("\r\n INNER JOIN {0} ON {1} = {2} AND {3} = {4} AND {5} = {6}",
                   SqlCommandHelper.EscapeTableName(PmxStockTransferRequestLineTable.NAME),
                   SqlCommandHelper.EscapeTableColumnName(PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseEntry.NAME),
                   SqlCommandHelper.EscapeTableColumnName(PmxStockTransferRequestLineTable.NAME, PmxStockTransferRequestLineTable.Columns.DocEntry.NAME),
                   SqlCommandHelper.EscapeTableColumnName(PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseLine.NAME),
                   SqlCommandHelper.EscapeTableColumnName(PmxStockTransferRequestLineTable.NAME, PmxStockTransferRequestLineTable.Columns.LineNum.NAME),
                   SqlCommandHelper.EscapeTableColumnName(PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseType.NAME),
                   SqlCommandHelper.FormatAndEscapeColumnValue(SboObjectTypes.InventoryTransferRequest, cultureDatabase));


                strBuilder.AppendFormat("\r\n LEFT OUTER JOIN {0} ON {1} = {2} AND {3} = {4}", // TBA make it possible the flow also sugests a location when the picklist is not based on a production order
                    SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxStockTransferRequestLineTable.NAME, PmxStockTransferRequestLineTable.Columns.ProductionOrderDocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.LineNum.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxStockTransferRequestLineTable.NAME, PmxStockTransferRequestLineTable.Columns.ProductionOrderLineNum.NAME));


                strBuilder.AppendFormat("\r\n LEFT JOIN {0} ON {1} = {2} AND {3} = {4}",
                    SqlCommandHelper.EscapeTableName(PmxLinedUpLocationTable.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxLinedUpLocationTable.NAME, PmxLinedUpLocationTable.Columns.ProductionOrderDocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxLinedUpLocationTable.NAME, PmxLinedUpLocationTable.Columns.ProductionOrderLineNum.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.LineNum.NAME));


                strBuilder.AppendFormat("\r\n WHERE {0} = {1} ",
                    SqlCommandHelper.EscapeTableColumnName(PmxPickListTable.NAME, PmxPickListTable.Columns.DocStatus.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(SboDocStatusColumnValues.Open.VALUE, cultureDatabase));


                strBuilder.AppendFormat("\r\n AND {0} = {1}",
                    SqlCommandHelper.EscapeTableColumnName(PmxMoveLineTable.NAME, PmxMoveLineTable.Columns.DestinationLogisticUnitIdentificationKey.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(luid, cultureDatabase));





                return strBuilder.ToString();
            }

            public static string GetProductionLine(int docEntry, CultureInfo cultureInfo)
            {
                StringBuilder strBuilder = new StringBuilder(254);

                strBuilder.AppendFormat("SELECT {0}, {1}, {2}, {3} FROM {4}",
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.ProductionLineCode.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.LineNum.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.HasToBeLinedUp.NAME),
                    SqlCommandHelper.EscapeTableName(PmxProductionOrderTable.NAME));

                strBuilder.AppendFormat("\r\nINNER JOIN {0} ON {1} = {2}", 
                    SqlCommandHelper.EscapeTableName(PmxProductionOrderLineTable.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderLineTable.NAME, PmxProductionOrderLineTable.Columns.DocEntry.NAME));

                strBuilder.AppendFormat("\r\nWHERE {0} IN (",
                    SqlCommandHelper.EscapeTableColumnName(PmxProductionOrderTable.NAME, PmxProductionOrderTable.Columns.DocEntry.NAME));

                strBuilder.Append("SELECT DISTINCT ");

                SqlCommandHelper.AddTableColumnNameWithAs(strBuilder, PmxStockTransferRequestLineTable.NAME, PmxStockTransferRequestLineTable.Columns.ProductionOrderDocEntry.NAME, PmxStockTransferRequestLineTable.Columns.ProductionOrderDocEntry.NAME);

                strBuilder.Append("\r\n FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxStockTransferRequestLineTable.NAME));

                strBuilder.Append("\r\n LEFT JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPickListProposalLineTable.NAME));

                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseEntry.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxStockTransferRequestLineTable.NAME, PmxStockTransferRequestLineTable.Columns.DocEntry.NAME);

                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseLine.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxStockTransferRequestLineTable.NAME, PmxStockTransferRequestLineTable.Columns.LineNum.NAME);

                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.BaseType.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxStockTransferRequestLineTable.NAME, PmxStockTransferRequestLineTable.Columns.ObjectType.NAME);

                strBuilder.Append("\r\n INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxPickListLineTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.BaseEntry.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.DocEntry.NAME);
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.BaseLine.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListProposalLineTable.NAME, PmxPickListProposalLineTable.Columns.LineNum.NAME);
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.BaseType.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(Produmex.Sbo.Logex.Data.BusinessObjects.Definitions.PmxObjectTypes.PmxPickListProposal, cultureInfo));

                strBuilder.Append("\r\n WHERE ");

                SqlCommandHelper.AddTableColumnName(strBuilder, PmxPickListLineTable.NAME, PmxPickListLineTable.Columns.DocEntry.NAME);
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(docEntry, cultureInfo));
                strBuilder.Append(")");

                return strBuilder.ToString();
            }

            public static string GetLinedUpLocationForProductionLine(string productionLine, string warehouseCode, string warehouseName, CultureInfo cultureInfo)
            {
                StringBuilder strBuilder = new StringBuilder(254);

                strBuilder.AppendFormat("SELECT {0}, {1}, {2} AS WarehouseCode, {3} AS WarehouseName FROM {4} WHERE {5} = {6}",
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxOseCanBeLinedUpLocationTable.NAME, PmxOseCanBeLinedUpLocationTable.Columns.CanBeLinedUpLocationCode.NAME, LocationDataSet.Columns.CODE.Name),
                    SqlCommandHelper.EscapeTableColumnNameWithAlias(PmxOseCanBeLinedUpLocationTable.NAME, PmxOseCanBeLinedUpLocationTable.Columns.CanBeLinedUpLocationCode.NAME, LocationDataSet.Columns.DESCRIPTION.Name),
                    SqlCommandHelper.FormatAndEscapeColumnValue(warehouseCode, cultureInfo),
                    SqlCommandHelper.FormatAndEscapeColumnValue(warehouseName, cultureInfo),
                    SqlCommandHelper.EscapeTableName(PmxOseCanBeLinedUpLocationTable.NAME),
                    SqlCommandHelper.EscapeTableColumnName(PmxOseCanBeLinedUpLocationTable.NAME, PmxOseCanBeLinedUpLocationTable.Columns.ParentCode.NAME),
                    SqlCommandHelper.FormatAndEscapeColumnValue(productionLine, cultureInfo));

                return strBuilder.ToString();
            }

            /// <summary>
            /// Command for getting all products order by name
            /// </summary>
            /// <param name="sscc">The SSCC.</param>
            /// <param name="locationCode">The location code.</param>
            /// <returns>
            /// Command for getting all products order by name
            /// </returns>
            public static string GetProductOnSSCC(string sscc, string locationCode, DbTool dbTool)
            {
                DbQueryBuilder strBuilder = dbTool.GetQueryBuilder();

                strBuilder.Append("SELECT ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.ItemCode.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.ItemName.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTable.NAME, PmxItemTable.Columns.DefaultStorageLocationOrZoneCode.NAME, PmxItemTable.Columns.DefaultStorageLocationOrZoneCode.NAME));


                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTable.NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.StorLocCode.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.LogisticCarrierCode.NAME, PmxLogisticUnitIDTable.Columns.LogisticCarrierCode.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME));

                strBuilder.Append(", ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.QualityStatusCode.NAME));


                strBuilder.AppendFormat(", " + strBuilder.IFNULL + "({0}, {1}) AS {2} ",
                  SqlCommandHelper.EscapeTableColumnName(SboUomTable.NAME, SboUomTable.Columns.UomCode.NAME),
                  SqlCommandHelper.EscapeTableColumnName(SboItemTable.NAME, SboItemTable.Columns.InventoryUom.NAME),
                  SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.InventoryUom.NAME));


                strBuilder.Append(", ");
                strBuilder.Append(SboRecordsetHelper.GetSelectPartsColumnAndIsColumnNull(PmxItemTable.NAME, PmxItemTable.Columns.UOM2.NAME, PmxInventoryTotalTable.Columns.Uom2.NAME));



                strBuilder.Append(", SUM( ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.Quantity.NAME);
                strBuilder.Append(" ) AS ");
                strBuilder.Append(strBuilder.AsIdentifier(PmxInventoryTotalTable.Columns.Quantity.NAME));


                strBuilder.Append(", SUM( ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QuantityUom2.NAME);
                strBuilder.Append(" ) AS ");
                strBuilder.Append(strBuilder.AsIdentifier(PmxInventoryTotalTable.Columns.QuantityUom2.NAME));


                strBuilder.Append("\r\n FROM ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));

                strBuilder.Append("\r\n INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));
                strBuilder.Append(" ON ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(SboItemTable.Columns.ItemCode.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxInventoryTotalTable.Columns.ItemCode.NAME));

                strBuilder.Append("\r\n INNER JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxLogisticUnitIDTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.InternalKey.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME);


                strBuilder.Append(" LEFT JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(SboUomTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, SboItemTable.Columns.InventoryUoMEntry.NAME);
                strBuilder.Append(" AND ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboUomTable.NAME, SboUomTable.Columns.UomEntry.NAME);
                strBuilder.Append(" <> -1 ");



                strBuilder.Append("\r\n LEFT JOIN ");
                strBuilder.Append(SqlCommandHelper.EscapeTableName(PmxItemTransactionalInfoTable.NAME));
                strBuilder.Append(" ON ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME);
                strBuilder.Append(" = ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.InternalKey.NAME);


                strBuilder.Append("\r\n WHERE ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxLogisticUnitIDTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxLogisticUnitIDTable.Columns.SSCC.NAME));
                strBuilder.Append(" = ");
                strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(sscc, CultureInfo.InvariantCulture));

                if (!string.IsNullOrEmpty(locationCode))
                {
                    strBuilder.Append(" AND ");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.NAME));
                    strBuilder.Append(".");
                    strBuilder.Append(SqlCommandHelper.EscapeColumnName(PmxInventoryTotalTable.Columns.StorLocCode.NAME));
                    strBuilder.Append(" = ");
                    strBuilder.Append(SqlCommandHelper.FormatAndEscapeColumnValue(locationCode, CultureInfo.InvariantCulture));
                }


                strBuilder.Append("\r\n GROUP BY ");

                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemCode.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, SboItemTable.Columns.ItemName.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.DefaultStorageLocationOrZoneCode.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.StorLocCode.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BatchNumber1.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTransactionalInfoTable.NAME, PmxItemTransactionalInfoTable.Columns.BestBeforeDate.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.ItemTransactionalInfoKey.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.LogUnitIdentKey.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxInventoryTotalTable.NAME, PmxInventoryTotalTable.Columns.QualityStatusCode.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboItemTable.NAME, SboItemTable.Columns.InventoryUom.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.UOM2.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxLogisticUnitIDTable.NAME, PmxLogisticUnitIDTable.Columns.LogisticCarrierCode.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, PmxItemTable.NAME, PmxItemTable.Columns.IsLogisticCarrier.NAME);
                strBuilder.Append(", ");
                SqlCommandHelper.AddTableColumnName(strBuilder, SboUomTable.NAME, SboUomTable.Columns.UomCode.NAME);


                strBuilder.Append("\r\n ORDER BY ");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.NAME));
                strBuilder.Append(".");
                strBuilder.Append(SqlCommandHelper.EscapeColumnName(SboItemTable.Columns.ItemName.NAME));

                return strBuilder.ToString();
            }
        }
        #endregion
    }
}
